@@ -12,6 +12,7 @@
 
 import unittest
 import os
+import pathlib
 import platform
 import warnings
 from io import StringIO, BytesIO
@@ -19,6 +20,7 @@ from urllib.error import URLError
 from urllib.request import urlopen
 from urllib.parse import urlsplit, uses_relative
 from pathlib import Path, PureWindowsPath, PurePath
+from unittest.mock import patch, MagicMock
 
 try:
     import lxml.etree as lxml_etree
@@ -34,11 +36,11 @@ from xmlschema.resources import is_url, is_local_url, is_remote_url, \
 from xmlschema.testing import SKIP_REMOTE_TESTS
 
 
-TEST_CASES_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_cases/')
+TEST_CASES_DIR = str(pathlib.Path(__file__).absolute().parent.joinpath('test_cases'))
 
 
 def casepath(relative_path):
-    return os.path.join(TEST_CASES_DIR, relative_path)
+    return str(pathlib.Path(TEST_CASES_DIR).joinpath(relative_path))
 
 
 def is_windows_path(path):
@@ -172,7 +174,9 @@ class TestResources(unittest.TestCase):
     def test_is_url_function(self):
         self.assertTrue(is_url(self.col_xsd_file))
         self.assertFalse(is_url('http://example.com['))
+        self.assertTrue(is_url(b'http://example.com'))
         self.assertFalse(is_url(' \t<root/>'))
+        self.assertFalse(is_url(b'  <root/>'))
         self.assertFalse(is_url('line1\nline2'))
         self.assertFalse(is_url(None))
 
@@ -194,9 +198,13 @@ class TestResources(unittest.TestCase):
 
     def test_url_path_is_file_function(self):
         self.assertTrue(url_path_is_file(self.col_xml_file))
+        self.assertTrue(url_path_is_file(normalize_url(self.col_xml_file)))
         self.assertFalse(url_path_is_file(self.col_dir))
         self.assertFalse(url_path_is_file('http://example.com/'))
 
+        with patch('platform.system', MagicMock(return_value="Windows")):
+            self.assertFalse(url_path_is_file('file:///c:/Windows/unknown'))
+
     def test_normalize_locations_function(self):
         locations = normalize_locations(
             [('tns0', 'alpha'), ('tns1', 'http://example.com/beta')], base_url='/home/user'
@@ -275,6 +283,7 @@ class TestResources(unittest.TestCase):
         self.assertEqual(resource.source, self.vh_xml_file)
         self.assertEqual(resource.root.tag, '{http://example.com/vehicles}vehicles')
         self.check_url(resource.url, self.vh_xml_file)
+        self.assertTrue(resource.filepath.endswith('vehicles.xml'))
         self.assertIsNone(resource.text)
         with self.assertRaises(XMLResourceError) as ctx:
             resource.load()
@@ -302,6 +311,7 @@ class TestResources(unittest.TestCase):
         self.assertEqual(resource.source, vh_etree)
         self.assertEqual(resource.root.tag, '{http://example.com/vehicles}vehicles')
         self.assertIsNone(resource.url)
+        self.assertIsNone(resource.filepath)
         self.assertIsNone(resource.text)
         resource.load()
         self.assertIsNone(resource.text)
@@ -310,6 +320,7 @@ class TestResources(unittest.TestCase):
         self.assertEqual(resource.source, vh_root)
         self.assertEqual(resource.root.tag, '{http://example.com/vehicles}vehicles')
         self.assertIsNone(resource.url)
+        self.assertIsNone(resource.filepath)
         self.assertIsNone(resource.text)
         resource.load()
         self.assertIsNone(resource.text)
@@ -323,6 +334,7 @@ class TestResources(unittest.TestCase):
         self.assertEqual(resource.source, vh_etree)
         self.assertEqual(resource.root.tag, '{http://example.com/vehicles}vehicles')
         self.assertIsNone(resource.url)
+        self.assertIsNone(resource.filepath)
         self.assertIsNone(resource.text)
         resource.load()
         self.assertIsNone(resource.text)
@@ -331,6 +343,7 @@ class TestResources(unittest.TestCase):
         self.assertEqual(resource.source, vh_root)
         self.assertEqual(resource.root.tag, '{http://example.com/vehicles}vehicles')
         self.assertIsNone(resource.url)
+        self.assertIsNone(resource.filepath)
         self.assertIsNone(resource.text)
         resource.load()
         self.assertIsNone(resource.text)
@@ -440,6 +453,25 @@ class TestResources(unittest.TestCase):
         resource.load()
         self.assertEqual(resource.text, source)
 
+    def test_xml_resource_from_malformed_source(self):
+        # related to issue #224
+        malformed_xml_file = casepath('resources/malformed.xml')
+        with self.assertRaises(ElementTree.ParseError):
+            XMLResource(malformed_xml_file)
+
+        with self.assertRaises(ElementTree.ParseError):
+            XMLResource(malformed_xml_file, defuse='always')
+
+        # the incremental parser does not found the incomplete root before the end
+        resource = XMLResource(malformed_xml_file, lazy=True)
+        self.assertEqual(resource.root.tag, 'malformed_xml_file')
+
+        resource = XMLResource('<malformed_xml_file>>', lazy=True)
+        self.assertEqual(resource.root.tag, 'malformed_xml_file')
+
+        with self.assertRaises(ElementTree.ParseError):
+            XMLResource('<malformed_xml_file<>', lazy=True)
+
     def test_xml_resource_from_wrong_arguments(self):
         self.assertRaises(TypeError, XMLResource, [b'<UNSUPPORTED_DATA_TYPE/>'])
 
@@ -1060,10 +1092,15 @@ class TestResources(unittest.TestCase):
                      "Remote networks are not accessible or avoid SSL "
                      "verification error on Windows.")
     def test_remote_resource_loading(self):
-        with urlopen("https://raw.githubusercontent.com/brunato/xmlschema/master/"
-                     "tests/test_cases/examples/collection/collection.xsd") as rh:
+        url = "https://raw.githubusercontent.com/brunato/xmlschema/master/" \
+              "tests/test_cases/examples/collection/collection.xsd"
+
+        with urlopen(url) as rh:
             col_xsd_resource = XMLResource(rh)
 
+        self.assertEqual(col_xsd_resource.url, url)
+        self.assertIsNone(col_xsd_resource.filepath)
+
         self.assertEqual(col_xsd_resource.namespace, XSD_NAMESPACE)
         self.assertIsNone(col_xsd_resource.seek(0))
         col_xsd_resource.load()
@@ -1087,7 +1124,7 @@ class TestResources(unittest.TestCase):
         self.assertTrue(isinstance(vh_schema, XMLSchema))
 
         xsd_source = """
-        <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" 
+        <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
                 xmlns:vh="http://example.com/vehicles">
             <xs:import namespace="http://example.com/vehicles" schemaLocation="{}"/>
         </xs:schema>""".format(self.vh_xsd_file)
@@ -1213,6 +1250,14 @@ class TestResources(unittest.TestCase):
         subresource = resource.subresource(root[0])
         self.assertIs(subresource.root, resource.root[0])
 
+    def test_loading_from_unrelated_dirs__issue_237(self):
+        relpath = str(pathlib.Path(__file__).parent.joinpath(
+            'test_cases/issues/issue_237/dir1/issue_237.xsd'
+        ))
+        schema = XMLSchema(relpath)
+        self.assertEqual(schema.maps.namespaces[''][1].name, 'issue_237a.xsd')
+        self.assertEqual(schema.maps.namespaces[''][2].name, 'issue_237b.xsd')
+
 
 if __name__ == '__main__':
     header_template = "Test xmlschema's XML resources with Python {} on platform {}"
