@@ -10,7 +10,9 @@
 #
 import unittest
 import os
+import json
 from decimal import Decimal
+from collections.abc import MutableMapping, MutableSequence, Set
 import base64
 
 try:
@@ -23,10 +25,11 @@ import xmlschema
 from xmlschema import XMLSchemaValidationError, ParkerConverter, BadgerFishConverter, \
     AbderaConverter, JsonMLConverter, ColumnarConverter
 
+from xmlschema.names import XSD_STRING
 from xmlschema.etree import ElementTree
 from xmlschema.converters import UnorderedConverter
 from xmlschema.validators import XMLSchema11
-from xmlschema.testing import XsdValidatorTestCase
+from xmlschema.testing import XsdValidatorTestCase, etree_elements_assert_equal
 
 VEHICLES_DICT = {
     '@xmlns:vh': 'http://example.com/vehicles',
@@ -316,6 +319,7 @@ DATA_DICT = {
         {'$': 'ISO-27001', '@Year': 2009}
     ],
     'decimal_value': [Decimal('1')],
+    'hexbin': 'AABBCCDD',
     'menù': 'baccalà mantecato',
     'complex_boolean': [
         {'$': True, '@Type': 2}, {'$': False, '@Type': 1}, True, False
@@ -325,6 +329,17 @@ DATA_DICT = {
 }
 
 
+def iter_nested_iterables(obj):
+    if not isinstance(obj, (MutableMapping, MutableSequence, Set, tuple)):
+        yield obj
+    else:
+        for item in obj.values() if isinstance(obj, MutableMapping) else obj:
+            if not isinstance(item, (MutableMapping, MutableSequence, Set, tuple)):
+                yield item
+            else:
+                yield from iter_nested_iterables(item)
+
+
 class TestDecoding(XsdValidatorTestCase):
     TEST_CASES_DIR = os.path.join(os.path.dirname(__file__), '../test_cases')
 
@@ -568,6 +583,48 @@ class TestDecoding(XsdValidatorTestCase):
         xd = self.vh_schema.to_dict(xt, '/vh:vehicles/vh:bikes', namespaces=self.vh_namespaces)
         self.assertEqual(xd['vh:bike'], VEHICLES_DICT['vh:bikes']['vh:bike'])
 
+    def test_max_depth_argument(self):
+        schema = self.schema_class(self.col_xsd_file)
+        self.assertEqual(
+            schema.decode(self.col_xml_file, max_depth=1),
+            {'@xmlns:col': 'http://example.com/ns/collection',
+             '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+             '@xsi:schemaLocation': 'http://example.com/ns/collection collection.xsd'})
+
+        xmlschema.limits.MAX_XML_DEPTH = 1
+        with self.assertRaises(XMLSchemaValidationError):
+            schema.decode(self.col_xml_file)
+        xmlschema.limits.MAX_XML_DEPTH = 9999
+
+        self.assertEqual(
+            schema.decode(self.col_xml_file, max_depth=2),
+            {'@xmlns:col': 'http://example.com/ns/collection',
+             '@xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+             '@xsi:schemaLocation': 'http://example.com/ns/collection collection.xsd',
+             'object': [{'@id': 'b0836217462', '@available': True},
+                        {'@id': 'b0836217463', '@available': True}]})
+
+    def test_value_hook_argument(self):
+        schema = self.schema_class(self.col_xsd_file)
+
+        def ascii_strings(value, xsd_type):
+            try:
+                if not isinstance(value, str) or \
+                        xsd_type.primitive_type.name != XSD_STRING:
+                    return value
+            except AttributeError:
+                return value
+            else:
+                return value.encode('utf-8')
+
+        obj = schema.decode(self.col_xml_file, value_hook=ascii_strings)
+        self.assertNotEqual(obj, schema.decode(self.col_xml_file))
+        root = schema.encode(obj)
+        self.assertIsNone(etree_elements_assert_equal(
+            root, ElementTree.parse(self.col_xml_file).getroot(), strict=False
+        ))
+        self.assertEqual(obj['object'][0]['title'], b'The Umbrellas')
+
     def test_non_global_schema_path(self):
         # Issue #157
         xs = xmlschema.XMLSchema("""<?xml version="1.0" encoding="UTF-8"?>
@@ -634,6 +691,11 @@ class TestDecoding(XsdValidatorTestCase):
         self.assertEqual(xs.decode('<dt>2001-04-15</dt>', datetime_types=True),
                          datatypes.Date10.fromstring('2001-04-15'))
 
+        xs = self.get_schema('<xs:attribute name="dt" type="xs:date"/>')
+        self.assertEqual(xs.attributes['dt'].decode('2001-04-15'), '2001-04-15')
+        self.assertEqual(xs.attributes['dt'].decode('2001-04-15', datetime_types=True),
+                         datatypes.Date10.fromstring('2001-04-15'))
+
     def test_duration_type(self):
         xs = self.get_schema('<xs:element name="td" type="xs:duration"/>')
         self.assertEqual(xs.decode('<td>P5Y3MT60H30.001S</td>'), 'P5Y3MT60H30.001S')
@@ -745,31 +807,86 @@ class TestDecoding(XsdValidatorTestCase):
         name_type = self.st_schema.types['NameType']
         self.check_decode(name_type, '', XMLSchemaValidationError)
 
-    def test_binary_data_facets(self):
+    def test_hex_binary_type(self):
         hex_code_type = self.st_schema.types['hexCode']
-        self.check_decode(hex_code_type, u'00D7310A', u'00D7310A')
+        self.check_decode(hex_code_type, '00D7310A', datatypes.HexBinary(b'00D7310A'))
+        self.check_decode(hex_code_type, 'D7310A', XMLSchemaValidationError)
 
+        xs = self.get_schema('<xs:element name="hex" type="xs:hexBinary"/>')
+
+        obj = xs.decode('<hex> 9AFD </hex>')
+        self.assertEqual(obj, '9AFD')  # Value string is collapsed anyway
+        self.assertIsInstance(obj, str)
+
+        obj = xs.decode('<hex> 9AFD </hex>', binary_types=True)
+        self.assertEqual(obj, '9AFD')
+        self.assertIsInstance(obj, datatypes.HexBinary)
+
+        xs = self.get_schema('<xs:attribute name="hex" type="xs:hexBinary"/>')
+
+        obj = xs.attributes['hex'].decode(' 9AFD ')
+        self.assertEqual(obj, '9AFD')
+        self.assertIsInstance(obj, str)
+
+        obj = xs.attributes['hex'].decode(' 9AFD ', binary_types=True)
+        self.assertEqual(obj, '9AFD')
+        self.assertIsInstance(obj, datatypes.HexBinary)
+
+    def test_base64_binary_type(self):
         base64_code_type = self.st_schema.types['base64Code']
         self.check_decode(base64_code_type, base64.b64encode(b'ok'), XMLSchemaValidationError)
         base64_value = base64.b64encode(b'hello')
-        self.check_decode(base64_code_type, base64_value, base64_value.decode('utf-8'))
-        self.check_decode(base64_code_type, base64.b64encode(b'abcefgh'), u'YWJjZWZnaA==')
-        self.check_decode(base64_code_type, b' Y  W J j ZWZ\t\tn\na A= =', u'Y W J j ZWZ n a A= =')
-        self.check_decode(base64_code_type, u' Y  W J j ZWZ\t\tn\na A= =', u'Y W J j ZWZ n a A= =')
-        self.check_decode(base64_code_type, base64.b64encode(b'abcefghi'), u'YWJjZWZnaGk=')
+
+        expected_value = datatypes.Base64Binary(base64_value)
+        self.check_decode(base64_code_type, base64_value, expected_value)
+
+        # Attribute
+        xs = self.get_schema('<xs:attribute name="b64" type="xs:base64Binary"/>')
+
+        obj = xs.attributes['b64'].decode(base64_value.decode())
+        self.assertEqual(obj, expected_value)
+        self.assertIsInstance(obj, str)
+
+        obj = xs.attributes['b64'].decode(base64_value.decode(), binary_types=True)
+        self.assertEqual(obj, expected_value)
+        self.assertIsInstance(obj, datatypes.Base64Binary)
+
+        # Element
+        xs = self.get_schema('<xs:element name="b64" type="xs:base64Binary"/>')
+
+        obj = xs.decode('<b64>{}</b64>'.format(base64_value.decode()))
+        self.assertEqual(obj, expected_value)
+        self.assertIsInstance(obj, str)
+
+        obj = xs.decode('<b64>{}</b64>'.format(base64_value.decode()), binary_types=True)
+        self.assertEqual(obj, expected_value)
+        self.assertIsInstance(obj, datatypes.Base64Binary)
+
+        self.check_decode(base64_code_type, base64.b64encode(b'abcefgh'),
+                          datatypes.Base64Binary('YWJjZWZnaA=='))
+        self.check_decode(base64_code_type, b' Y  W J j ZWZ\t\tn\na A= =',
+                          datatypes.Base64Binary('Y W J j ZWZ n a A= ='))
+        self.check_decode(base64_code_type, u' Y  W J j ZWZ\t\tn\na A= =',
+                          datatypes.Base64Binary('Y W J j ZWZ n a A= ='))
+        self.check_decode(base64_code_type, base64.b64encode(b'abcefghi'),
+                          datatypes.Base64Binary('YWJjZWZnaGk='))
 
         self.check_decode(base64_code_type, u'YWJjZWZnaA=', XMLSchemaValidationError)
         self.check_decode(base64_code_type, u'YWJjZWZna$==', XMLSchemaValidationError)
 
         base64_length4_type = self.st_schema.types['base64Length4']
-        self.check_decode(base64_length4_type, base64.b64encode(b'abc'), XMLSchemaValidationError)
-        self.check_decode(base64_length4_type, base64.b64encode(b'abce'), u'YWJjZQ==')
-        self.check_decode(base64_length4_type, base64.b64encode(b'abcef'), XMLSchemaValidationError)
+        self.check_decode(base64_length4_type, base64.b64encode(b'abc'),
+                          XMLSchemaValidationError)
+        self.check_decode(base64_length4_type, base64.b64encode(b'abce'),
+                          datatypes.Base64Binary('YWJjZQ=='))
+        self.check_decode(base64_length4_type, base64.b64encode(b'abcef'),
+                          XMLSchemaValidationError)
 
         base64_length5_type = self.st_schema.types['base64Length5']
         self.check_decode(base64_length5_type, base64.b64encode(b'1234'),
                           XMLSchemaValidationError)
-        self.check_decode(base64_length5_type, base64.b64encode(b'12345'), u'MTIzNDU=')
+        self.check_decode(base64_length5_type, base64.b64encode(b'12345'),
+                          datatypes.Base64Binary('MTIzNDU='))
         self.check_decode(base64_length5_type, base64.b64encode(b'123456'),
                           XMLSchemaValidationError)
 
@@ -917,7 +1034,7 @@ class TestDecoding(XsdValidatorTestCase):
         data = schema.decode(self.casepath('issues/issue_204/issue_204_3.xml'),
                              validation='lax', keep_unknown=True)
         self.assertEqual(set(x for x in data[0] if x[0] != '@'), {'child2', 'unknown', 'child5'})
-        self.assertEqual(data[0]['unknown'], {'a': [{'$': '1'}], 'b': [None]})
+        self.assertEqual(data[0]['unknown'], {'a': ['1'], 'b': [None]})
 
         data = schema.decode(self.casepath('issues/issue_204/issue_204_2.xml'), validation='skip')
         self.assertEqual(set(x for x in data if x[0] != '@'), {'child2', 'child5'})
@@ -925,7 +1042,7 @@ class TestDecoding(XsdValidatorTestCase):
         data = schema.decode(self.casepath('issues/issue_204/issue_204_3.xml'),
                              validation='skip', keep_unknown=True)
         self.assertEqual(set(x for x in data if x[0] != '@'), {'child2', 'unknown', 'child5'})
-        self.assertEqual(data['unknown'], {'a': [{'$': '1'}], 'b': [None]})
+        self.assertEqual(data['unknown'], {'a': ['1'], 'b': [None]})
 
     def test_error_message__issue_115(self):
         schema = self.schema_class(self.casepath('issues/issue_115/Rotation.xsd'))
@@ -994,7 +1111,7 @@ class TestDecoding(XsdValidatorTestCase):
         schema = self.schema_class(self.casepath('issues/issue_190/issue_190.xsd'))
         self.assertEqual(
             schema.to_dict(self.casepath('issues/issue_190/issue_190.xml')),
-            {'a': {'c': [{'$': '1'}]}, 'b': {'c': [{'$': '1'}], 'e': [{'$': '1'}]}}
+            {'a': {'c': ['1']}, 'b': {'c': ['1'], 'e': ['1']}}
         )
 
     def test_issue_200(self):
@@ -1010,6 +1127,31 @@ class TestDecoding(XsdValidatorTestCase):
                            path='/na:main/na:item[@doc_id=2]'),
         self.assertIn('is not an element of the schema', str(ctx.exception))
 
+    def test_issue_238__decode_bytes_strings(self):
+        with open(self.vh_xml_file, 'rb') as fp:
+            vh_xml_data = fp.read()
+
+        self.assertIsInstance(vh_xml_data, bytes)
+
+        obj = self.vh_schema.decode(vh_xml_data)
+
+        # ElementTree accepts also bytes but emits Unicode strings only
+        for value in iter_nested_iterables(obj):
+            self.assertNotIsInstance(value, bytes)
+
+    def test_issue_240__decode_unicode_to_json(self):
+        schema = self.get_schema('<xs:element name="chars" type="xs:string"/>')
+
+        xml_data = '<chars>øæå</chars>'
+        obj = xmlschema.to_dict(xml_data, schema=schema, decimal_type=str)
+        self.assertEqual(obj, 'øæå')
+        self.assertEqual(obj, schema.decode(xml_data, decimal_type=str))
+
+        json_data = json.dumps(obj, indent=4)
+        self.assertEqual(obj, 'øæå')
+        self.assertIsInstance(obj, str)
+        self.assertEqual(json_data.encode("utf-8"), b'"\\u00f8\\u00e6\\u00e5"')
+
 
 class TestDecoding11(TestDecoding):
     schema_class = XMLSchema11
@@ -1025,6 +1167,11 @@ class TestDecoding11(TestDecoding):
         self.assertEqual(xs.decode('<dt>2001-04-15</dt>', datetime_types=True),
                          datatypes.Date.fromstring('2001-04-15'))
 
+        xs = self.get_schema('<xs:attribute name="dt" type="xs:date"/>')
+        self.assertEqual(xs.attributes['dt'].decode('2001-04-15'), '2001-04-15')
+        self.assertEqual(xs.attributes['dt'].decode('2001-04-15', datetime_types=True),
+                         datatypes.Date.fromstring('2001-04-15'))
+
     def test_derived_duration_types(self):
         xs = self.get_schema('<xs:element name="td" type="xs:yearMonthDuration"/>')
         self.assertEqual(xs.decode('<td>P0Y4M</td>'), 'P0Y4M')
