@@ -20,11 +20,11 @@ import re
 from textwrap import dedent
 
 from xmlschema import XMLSchemaParseError, XMLSchemaIncludeWarning, XMLSchemaImportWarning
-from xmlschema.names import LOCATION_HINTS, SCHEMAS_DIR, XSD_ELEMENT, XSI_TYPE
+from xmlschema.names import XML_NAMESPACE, LOCATION_HINTS, SCHEMAS_DIR, XSD_ELEMENT, XSI_TYPE
 from xmlschema.etree import etree_element
-from xmlschema.validators import XMLSchemaBase, XMLSchema11
+from xmlschema.validators import XMLSchemaBase, XMLSchema11, XsdGlobals
 from xmlschema.testing import SKIP_REMOTE_TESTS, XsdValidatorTestCase
-from xmlschema.validators.schema import logger
+from xmlschema.validators.schema import logger, XMLSchemaNotBuiltError
 
 
 class TestXMLSchema10(XsdValidatorTestCase):
@@ -55,13 +55,60 @@ class TestXMLSchema10(XsdValidatorTestCase):
         self.assertNotEqual(id(self.vh_schema.namespaces), id(schema.namespaces))
         self.assertNotEqual(id(self.vh_schema.maps), id(schema.maps))
 
+    def test_schema_location_hints(self):
+        with warnings.catch_warnings(record=True):
+            warnings.simplefilter("always")
+            schema = self.schema_class(dedent("""\
+                <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+                    xsi:schemaLocation="http://xmlschema.test/ns schema.xsd">
+                  <xs:element name="root" />
+                </xs:schema>"""))
+
+        self.assertEqual(schema.schema_location, [("http://xmlschema.test/ns", "schema.xsd")])
+        self.assertIsNone(schema.no_namespace_schema_location)
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+                xsi:noNamespaceSchemaLocation="schema.xsd">
+              <xs:element name="root" />
+            </xs:schema>"""))
+
+        self.assertEqual(schema.schema_location, [])
+        self.assertEqual(schema.no_namespace_schema_location, 'schema.xsd')
+
+    def test_target_prefix(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                targetNamespace="http://xmlschema.test/ns">
+              <xs:element name="root" />
+            </xs:schema>"""))
+
+        self.assertEqual(schema.target_prefix, '')
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                xmlns:tns="http://xmlschema.test/ns"
+                targetNamespace="http://xmlschema.test/ns">
+              <xs:element name="root" />
+            </xs:schema>"""))
+
+        self.assertEqual(schema.target_prefix, 'tns')
+
+    def test_builtin_types(self):
+        self.assertIn('string', self.schema_class.builtin_types())
+
+        with self.assertRaises(XMLSchemaNotBuiltError):
+            self.schema_class.meta_schema.builtin_types()
+
     def test_resolve_qname(self):
-        schema = self.schema_class("""<xs:schema
-            xmlns:xs="http://www.w3.org/2001/XMLSchema"
-            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">    
+              <xs:element name="root" />
+            </xs:schema>"""))
 
-            <xs:element name="root" />
-        </xs:schema>""")
         self.assertEqual(schema.resolve_qname('xs:element'), XSD_ELEMENT)
         self.assertEqual(schema.resolve_qname('xsi:type'), XSI_TYPE)
 
@@ -139,9 +186,21 @@ class TestXMLSchema10(XsdValidatorTestCase):
         </xs:simpleType>""", XMLSchemaParseError)
 
     def test_base_schemas(self):
-        self.schema_class(os.path.join(SCHEMAS_DIR, 'XML/xml_minimal.xsd'))
+        xsd_file = os.path.join(SCHEMAS_DIR, 'XML/xml_minimal.xsd')
+        schema = self.schema_class(xsd_file)
+        self.assertEqual(schema.target_namespace, XML_NAMESPACE)
 
     def test_root_elements(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"/>"""))
+        self.assertEqual(schema.root_elements, [])
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+              <xs:element name="root" />
+            </xs:schema>"""))
+        self.assertEqual(schema.root_elements, [schema.elements['root']])
+
         # Test issue #107 fix
         schema = self.schema_class(dedent("""\
             <?xml version="1.0" encoding="utf-8"?>
@@ -158,6 +217,14 @@ class TestXMLSchema10(XsdValidatorTestCase):
         self.assertEqual(set(schema.root_elements),
                          {schema.elements['root1'], schema.elements['root2']})
 
+    def test_simple_types(self):
+        self.assertListEqual(self.vh_schema.simple_types, [])
+        self.assertGreater(len(self.st_schema.simple_types), 20)
+
+    def test_complex_types(self):
+        self.assertListEqual(self.vh_schema.complex_types,
+                             [self.vh_schema.types['vehicleType']])
+
     def test_is_restriction_method(self):
         # Test issue #111 fix
         schema = self.schema_class(source=self.casepath('issues/issue_111/issue_111.xsd'))
@@ -349,16 +416,171 @@ class TestXMLSchema10(XsdValidatorTestCase):
             </xs:schema>"""))
         self.assertEqual(len(schema.root), 0 if schema.XSD_VERSION == '1.0' else 1)
 
+    def test_xsd_version_compatibility_property(self):
+        self.assertEqual(self.vh_schema.xsd_version, self.vh_schema.XSD_VERSION)
+
     def test_explicit_locations(self):
         source = dedent("""\
-                    <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
-                        <xs:element name="root"/>
-                    </xs:schema>""")
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="root"/>
+            </xs:schema>""")
 
         locations = {'http://example.com/vehicles': self.vh_xsd_file}
         schema = self.schema_class(source, locations=locations)
         self.assertEqual(len(schema.maps.namespaces['http://example.com/vehicles']), 4)
 
+    def test_use_meta_property(self):
+        self.assertTrue(self.vh_schema.use_meta)
+        self.assertTrue(self.col_schema.use_meta)
+
+        meta_schema = self.schema_class.meta_schema
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="foo"/>
+            </xs:schema>"""), use_meta=False)
+        self.assertIsNot(meta_schema, schema.meta_schema)
+        self.assertFalse(schema.use_meta)
+
+    def test_other_schema_root_attributes(self):
+        self.assertIsNone(self.vh_schema.id)
+        self.assertIsNone(self.vh_schema.version)
+
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" id="foo" version="2.0">
+                <xs:element name="foo"/>
+            </xs:schema>"""))
+        self.assertEqual(schema.id, 'foo')
+        self.assertEqual(schema.version, '2.0')
+
+    def test_change_maps_attribute(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="root"/>
+            </xs:schema>"""))
+
+        with self.assertRaises(ValueError) as ctx:
+            schema.meta_schema.maps = XsdGlobals(schema, schema.validation)
+        self.assertEqual(str(ctx.exception),
+                         "cannot change the global maps instance of a meta-schema")
+
+        self.assertTrue(schema.built)
+        maps, schema.maps = schema.maps, XsdGlobals(schema, schema.validation)
+        self.assertIsNot(maps, schema.maps)
+        self.assertFalse(schema.built)
+        schema.maps = maps
+        self.assertTrue(schema.built)
+
+    def test_listed_and_reversed_elements(self):
+        schema = self.schema_class(dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="elem1"/>
+                <xs:element name="elem2"/>
+                <xs:element name="elem3"/>
+            </xs:schema>"""))
+
+        elements = list(schema)
+        self.assertListEqual(elements, [schema.elements['elem1'],
+                                        schema.elements['elem2'],
+                                        schema.elements['elem3']])
+        elements.reverse()
+        self.assertListEqual(elements, list(reversed(schema)))
+
+    def test_multi_schema_initilization(self):
+        source1 = dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="elem1"/>
+            </xs:schema>""")
+
+        source2 = dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="elem2"/>
+            </xs:schema>""")
+
+        source3 = dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="elem3"/>
+            </xs:schema>""")
+
+        schema = self.schema_class([source1, source2, source3])
+        self.assertEqual(len(schema.elements), 3)
+        self.assertEqual(len(schema.maps.namespaces['']), 3)
+        self.assertIs(schema.elements['elem1'].schema, schema)
+        self.assertIs(schema.elements['elem2'].schema, schema.maps.namespaces[''][1])
+        self.assertIs(schema.elements['elem3'].schema, schema.maps.namespaces[''][2])
+
+        with self.assertRaises(XMLSchemaParseError) as ec:
+            self.schema_class([source1, source2, source2])
+        self.assertIn("global element with name='elem2' is already defined", str(ec.exception))
+
+        source1 = dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" 
+                    targetNamespace="http://xmlschema.test/ns">
+                <xs:element name="elem1"/>
+            </xs:schema>""")
+
+        schema = self.schema_class([source1, source2])
+        self.assertEqual(len(schema.elements), 2)
+        self.assertEqual(len(schema.maps.namespaces['http://xmlschema.test/ns']), 2)
+        self.assertIs(schema.elements['elem1'].schema, schema)
+        self.assertIs(schema.elements['elem2'].schema,
+                      schema.maps.namespaces['http://xmlschema.test/ns'][1])
+
+    def test_add_schema(self):
+        source1 = dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" 
+                    targetNamespace="http://xmlschema.test/ns">
+                <xs:element name="elem1"/>
+            </xs:schema>""")
+
+        source2 = dedent("""\
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:element name="elem2"/>
+            </xs:schema>""")
+
+        source3 = dedent("""\
+             <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" 
+                     targetNamespace="http://xmlschema.test/ns1">
+                 <xs:element name="elem3"/>
+             </xs:schema>""")
+
+        schema = self.schema_class(source1)
+        schema.add_schema(source2, build=True)
+        self.assertEqual(len(schema.elements), 1)
+        self.assertEqual(len(schema.maps.namespaces['http://xmlschema.test/ns']), 1)
+        self.assertEqual(len(schema.maps.namespaces['']), 1)
+
+        # Less checks on duplicate objects for schemas added after the build
+        schema.add_schema(source2, build=True)
+        self.assertEqual(len(schema.maps.namespaces['']), 2)
+        self.assertTrue(schema.maps.built)
+
+        with self.assertRaises(XMLSchemaParseError) as ec:
+            schema.maps.clear()
+            schema.build()
+        self.assertIn("global element with name='elem2' is already defined", str(ec.exception))
+
+        schema = self.schema_class(source1)
+        schema.add_schema(source2, namespace='http://xmlschema.test/ns', build=True)
+        self.assertEqual(len(schema.maps.namespaces['http://xmlschema.test/ns']), 2)
+
+        # Need a rebuild to add elem2 from added schema ...
+        self.assertEqual(len(schema.elements), 1)
+        schema.maps.clear()
+        schema.build()
+        self.assertEqual(len(schema.elements), 2)
+
+        # ... so is better to build after sources additions
+        schema = self.schema_class(source1, build=False)
+        schema.add_schema(source2, namespace='http://xmlschema.test/ns')
+        schema.build()
+        self.assertEqual(len(schema.elements), 2)
+
+        # Adding other namespaces do not require rebuild
+        schema3 = schema.add_schema(source3, build=True)
+        self.assertEqual(len(schema.maps.namespaces['http://xmlschema.test/ns1']), 1)
+        self.assertEqual(len(schema3.elements), 1)
+
     def test_export_errors__issue_187(self):
         with self.assertRaises(ValueError) as ctx:
             self.vh_schema.export(target=self.vh_dir)
@@ -464,7 +686,7 @@ class TestXMLSchema10(XsdValidatorTestCase):
         self.assertFalse(os.path.isdir(dirname))
 
         with tempfile.TemporaryDirectory() as dirname:
-            vh_schema.export(target=dirname, only_relative=False)
+            vh_schema.export(target=dirname, save_remote=True)
             path = pathlib.Path(dirname).joinpath('brunato/xmlschema/master/tests/test_cases/'
                                                   'examples/vehicles/*.xsd')
 
