@@ -11,9 +11,9 @@ import json
 from collections.abc import Iterator
 
 from .exceptions import XMLSchemaTypeError, XMLSchemaValueError, XMLResourceError
-from .namespaces import XSD_NAMESPACE
-from .etree import ElementTree, is_etree_document, etree_tostring
-from .qnames import XSI_TYPE
+from .names import XSD_NAMESPACE, XSI_TYPE
+from .etree import ElementTree, etree_tostring
+from .helpers import is_etree_document
 from .resources import fetch_schema_locations, XMLResource
 from .validators import XMLSchema10, XMLSchemaBase, XMLSchemaValidationError
 
@@ -289,7 +289,8 @@ def from_json(source, schema, path=None, converter=None, json_options=None, **kw
     :param converter: an :class:`XMLSchemaConverter` subclass or instance to use \
     for the encoding.
     :param json_options: a dictionary with options for the JSON deserializer.
-    :param kwargs: Keyword arguments containing options for converter and encoding.
+    :param kwargs: other optional arguments of :meth:`XMLSchema.iter_encode` and \
+    options for converter.
     :return: An element tree's Element instance. If ``validation='lax'`` keyword argument is \
     provided the validation errors are collected and returned coupled in a tuple with the \
     Element instance.
