@@ -8,12 +8,13 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 import os.path
+import platform
 import re
 from string import ascii_letters
 from elementpath import iter_select, XPath1Parser, XPathContext, XPath2Parser
 from io import StringIO, BytesIO
 from urllib.request import urlopen, pathname2url
-from urllib.parse import uses_relative, urlsplit, urljoin, urlunsplit
+from urllib.parse import uses_relative, urlsplit, urljoin, urlunsplit, unquote
 from urllib.error import URLError
 
 from .exceptions import XMLSchemaTypeError, XMLSchemaValueError, XMLResourceError
@@ -171,7 +172,10 @@ def is_url(obj):
     """
     if not isinstance(obj, (str, bytes)):
         return False
-    elif '\n' in obj or obj.lstrip().startswith('<'):
+    elif isinstance(obj, str):
+        if '\n' in obj or obj.lstrip().startswith('<'):
+            return False
+    elif b'\n' in obj or obj.lstrip().startswith(b'<'):
         return False
 
     try:
@@ -199,7 +203,12 @@ def is_local_url(url):
 def url_path_is_file(url):
     if not is_local_url(url):
         return False
-    return os.path.isfile(url) or os.path.isfile(urlsplit(normalize_url(url)).path)
+    if os.path.isfile(url):
+        return True
+    path = unquote(urlsplit(normalize_url(url)).path)
+    if path.startswith('/') and platform.system() == 'Windows':
+        path = path[1:]
+    return os.path.isfile(path)
 
 
 ###
@@ -402,10 +411,17 @@ class XMLResource:
         """The XML text source, `None` if it's not available."""
         return self._text
 
+    @property
+    def name(self):
+        """
+        The source name, is `None` if the instance is created from an Element or a string.
+        """
+        return os.path.basename(self._url) if self.url else None
+
     @property
     def url(self):
         """
-        The source URL, `None` if the instance is created from an Element tree or from a string.
+        The source URL, `None` if the instance is created from an Element or a string.
         """
         return self._url
 
@@ -414,6 +430,16 @@ class XMLResource:
         """The effective base URL used for completing relative locations."""
         return os.path.dirname(self._url) if self._url else self._base_url
 
+    @property
+    def filepath(self):
+        """
+        The resource filepath if the intance is created from a local file, `None` otherwise.
+        """
+        if self._url:
+            url_parts = urlsplit(self._url)
+            if url_parts.scheme in ('', 'file'):
+                return url_parts.path
+
     @property
     def allow(self):
         """The security mode for accessing resource locations."""
@@ -584,10 +610,10 @@ class XMLResource:
                 self._text = None
                 self._lazy = lazy
 
-        elif isinstance(source, str):
+        elif isinstance(source, (str, bytes)):
             # source is a string containing an XML document
             _url, self._url = self._url, None
-            resource = StringIO(source)
+            resource = StringIO(source) if isinstance(source, str) else BytesIO(source)
             try:
                 if not lazy:
                     self._parse(resource)
@@ -733,7 +759,7 @@ class XMLResource:
             msg = "{!r} is not an element or the XML resource tree"
             raise XMLResourceError(msg.format(elem))
 
-        resource = XMLResource(elem, self._base_url, self._allow, self._defuse, self._timeout)
+        resource = XMLResource(elem, self.base_url, self._allow, self._defuse, self._timeout)
         if not hasattr(elem, 'nsmap'):
             namespaces = {}
             _nsmap = self._nsmap[elem]
@@ -810,8 +836,6 @@ class XMLResource:
         resource = self.open()
         try:
             data = resource.read()
-        except (OSError, IOError) as err:
-            raise XMLResourceError("cannot load data from %r: %s" % (self._url, err))
         finally:
             # We don't want to close the file obj if it wasn't originally
             # opened by `XMLResource`. That is the concern of the code
