@@ -12,7 +12,7 @@ This module contains classes for XML Schema attributes and attribute groups.
 """
 from decimal import Decimal
 from collections.abc import MutableMapping
-from elementpath.datatypes import AbstractDateTime, Duration
+from elementpath.datatypes import AbstractDateTime, Duration, AbstractBinary
 from typing import Union, Dict, Optional
 
 from ..exceptions import XMLSchemaValueError
@@ -58,7 +58,7 @@ class XsdAttribute(XsdComponent, ValidationMixin):
     inheritable = False  # For XSD 1.1 attributes, always False for XSD 1.0 attributes.
 
     def _parse(self):
-        super(XsdAttribute, self)._parse()
+        super()._parse()
         attrib = self.elem.attrib
 
         if 'use' in attrib and self.parent is not None and \
@@ -195,6 +195,9 @@ class XsdAttribute(XsdComponent, ValidationMixin):
     def is_prohibited(self):
         return self.use == 'prohibited'
 
+    def is_empty(self):
+        return self.fixed == '' or self.type.is_empty()
+
     def iter_components(self, xsd_classes=None):
         if xsd_classes is None or isinstance(self, xsd_classes):
             yield self
@@ -226,34 +229,37 @@ class XsdAttribute(XsdComponent, ValidationMixin):
                 msg = "attribute {!r} has a fixed value {!r}".format(self.name, self.fixed)
                 yield self.validation_error(validation, msg, text, **kwargs)
 
-        for result in self.type.iter_decode(text, validation, **kwargs):
-            if isinstance(result, XMLSchemaValidationError):
-                result.reason = 'attribute {}={!r}: {}'.format(
-                    self.prefixed_name, text, result.reason
+        for value in self.type.iter_decode(text, validation, **kwargs):
+            if isinstance(value, XMLSchemaValidationError):
+                value.reason = 'attribute {}={!r}: {}'.format(
+                    self.prefixed_name, text, value.reason
                 )
-                yield result
+                yield value
                 continue
-            elif isinstance(result, Decimal):
-                try:
-                    yield kwargs['decimal_type'](result)
-                except (KeyError, TypeError):
-                    yield result
-            elif isinstance(result, (AbstractDateTime, Duration)):
-                try:
-                    yield result if kwargs['datetime_types'] is True else text
-                except KeyError:
+            elif 'value_hook' in kwargs:
+                yield kwargs['value_hook'](value, self.type)
+            elif isinstance(value, (int, float, list)) or value is None:
+                yield value
+            elif isinstance(value, str):
+                if value.startswith('{') and self.type.is_qname():
                     yield text
-            elif isinstance(result, str) and result.startswith('{') and self.type.is_qname():
-                yield text
+                else:
+                    yield value
+            elif isinstance(value, Decimal):
+                try:
+                    yield kwargs['decimal_type'](value)
+                except (KeyError, TypeError):
+                    yield value
+            elif isinstance(value, (AbstractDateTime, Duration)):
+                yield value if kwargs.get('datetime_types') else text.strip()
+            elif isinstance(value, AbstractBinary) and not kwargs.get('binary_types'):
+                yield str(value)
             else:
-                yield result
-            break
+                yield value
+            break  # pragma: no cover
 
     def iter_encode(self, obj, validation='lax', **kwargs):
-        for result in self.type.iter_encode(obj, validation):
-            yield result
-            if not isinstance(result, XMLSchemaValidationError):
-                return
+        yield from self.type.iter_encode(obj, validation, **kwargs)
 
 
 class Xsd11Attribute(XsdAttribute):
@@ -287,7 +293,7 @@ class Xsd11Attribute(XsdAttribute):
             return self.schema.target_namespace
 
     def _parse(self):
-        super(Xsd11Attribute, self)._parse()
+        super()._parse()
         if self.use == 'prohibited' and 'fixed' in self.elem.attrib:
             self.parse_error("attribute 'fixed' with use=prohibited is not allowed in XSD 1.1")
         if 'inheritable' in self.elem.attrib:
@@ -320,9 +326,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
         XsdComponent.__init__(self, elem, schema, parent)
 
     def __repr__(self):
-        if self.ref is not None:
-            return '%s(ref=%r)' % (self.__class__.__name__, self.name)
-        elif self.name is not None:
+        if self.name is not None:
             return '%s(name=%r)' % (self.__class__.__name__, self.name)
         elif self:
             names = [a if a.name is None else a.name for a in self.values()]
@@ -335,19 +339,9 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
         return self._attribute_group[key]
 
     def __setitem__(self, key, value: Union[XsdAttribute, XsdAnyAttribute]):
-        if key is None:
-            self._attribute_group[key] = value
-        else:
-            if key[0] != '{':
-                if value.local_name != key:
-                    raise XMLSchemaValueError("%r name and key %r mismatch." % (value.name, key))
-                if value.target_namespace != self.target_namespace:
-                    # Qualify attributes of other namespaces
-                    key = value.qualified_name
-            elif value.qualified_name != key:
-                raise XMLSchemaValueError("%r name and key %r mismatch." % (value.name, key))
-
-            self._attribute_group[key] = value
+        if value.name != key:
+            raise XMLSchemaValueError("%r name and key %r mismatch" % (value.name, key))
+        self._attribute_group[key] = value
 
     def __delitem__(self, key):
         del self._attribute_group[key]
@@ -363,27 +357,26 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
         return len(self._attribute_group)
 
     def _parse(self):
-        super(XsdAttributeGroup, self)._parse()
-        any_attribute = None
-        attribute_group_refs = []
+        super()._parse()
 
         if self.elem.tag == XSD_ATTRIBUTE_GROUP:
             if self.parent is not None:
-                return  # Skip dummy definitions
+                return  # Skip parsing dummy instances
             try:
                 self.name = get_qname(self.target_namespace, self.elem.attrib['name'])
             except KeyError:
-                self.parse_error("an attribute group declaration requires a 'name' attribute.")
                 return
             else:
                 if self.schema.default_attributes == self.name and self.xsd_version > '1.0':
                     self.schema.default_attributes = self
 
+        any_attribute = None
+        attribute_group_refs = []
         attributes: Dict[Optional[str], Union[XsdAttribute, XsdAnyAttribute]] = {}
 
         for child in self.elem:
             if child.tag == XSD_ANNOTATION or callable(child.tag):
-                continue
+                continue  # pragma: no cover
             elif any_attribute is not None:
                 if child.tag == XSD_ANY_ATTRIBUTE:
                     self.parse_error("more anyAttribute declarations in the same attribute group")
@@ -399,16 +392,12 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                     attributes[None] = any_attribute
 
             elif child.tag == XSD_ATTRIBUTE:
-                try:
-                    attribute = self.schema.BUILDERS.attribute_class(child, self.schema, self)
-                except TypeError as err:
-                    self.parse_error(err, elem=child)
-                else:
-                    if attribute.name in attributes:
-                        self.parse_error("multiple declaration for attribute "
-                                         "{!r}".format(attribute.name))
-                    elif attribute.use != 'prohibited' or self.elem.tag != XSD_ATTRIBUTE_GROUP:
-                        attributes[attribute.name] = attribute
+                attribute = self.schema.BUILDERS.attribute_class(child, self.schema, self)
+                if attribute.name in attributes:
+                    self.parse_error("multiple declaration for attribute "
+                                     "{!r}".format(attribute.name))
+                elif attribute.use != 'prohibited' or self.elem.tag != XSD_ATTRIBUTE_GROUP:
+                    attributes[attribute.name] = attribute
 
             elif child.tag == XSD_ATTRIBUTE_GROUP:
                 try:
@@ -566,7 +555,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
 
     def parse_error(self, error, elem=None, validation=None):
         if self.parent is None:
-            super(XsdAttributeGroup, self).parse_error(error, elem, validation)
+            super().parse_error(error, elem, validation)
         else:
             self.parent.parse_error(error, elem, validation)
 
@@ -592,10 +581,10 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
     def iter_components(self, xsd_classes=None):
         if xsd_classes is None or isinstance(self, xsd_classes):
             yield self
-        if self.ref is None:
-            for attr in self.values():
-                if attr.parent is not None:
-                    yield from attr.iter_components(xsd_classes)
+
+        for attr in self.values():
+            if attr.parent is not None:
+                yield from attr.iter_components(xsd_classes)
 
     def iter_decode(self, attrs, validation='lax', **kwargs):
         if not attrs and not self:
@@ -606,7 +595,10 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
             yield self.validation_error(validation, reason, attrs, **kwargs)
 
         kwargs['level'] = kwargs.get('level', 0) + 1
-        use_defaults = kwargs.get('use_defaults', True)
+        try:
+            use_defaults = kwargs['use_defaults']
+        except KeyError:
+            use_defaults = True
 
         additional_attrs = [
             (k, v) for k, v in self.iter_value_constraints(use_defaults) if k not in attrs
@@ -622,14 +614,14 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
         result_list = []
         for name, value in attrs.items():
             try:
-                xsd_attribute = self[name]
+                xsd_attribute = self._attribute_group[name]
             except KeyError:
                 if get_namespace(name) == XSI_NAMESPACE:
                     try:
                         xsd_attribute = self.maps.lookup_attribute(name)
                     except LookupError:
                         try:
-                            xsd_attribute = self[None]  # None key ==> anyAttribute
+                            xsd_attribute = self._attribute_group[None]  # None == anyAttribute
                             value = (name, value)
                         except KeyError:
                             reason = "%r is not an attribute of the XSI namespace." % name
@@ -637,7 +629,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                             continue
                 else:
                     try:
-                        xsd_attribute = self[None]  # None key ==> anyAttribute
+                        xsd_attribute = self._attribute_group[None]  # None == anyAttribute
                         value = (name, value)
                     except KeyError:
                         reason = "%r attribute not allowed for element." % name
@@ -645,7 +637,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                         continue
             else:
                 if xsd_attribute.use == 'prohibited' and \
-                        (None not in self or not self[None].is_matching(name)):
+                        (None not in self or not self._attribute_group[None].is_matching(name)):
                     reason = "use of attribute %r is prohibited" % name
                     yield self.validation_error(validation, reason, attrs, **kwargs)
 
@@ -685,7 +677,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
         result_list = []
         for name, value in attrs.items():
             try:
-                xsd_attribute = self[name]
+                xsd_attribute = self._attribute_group[name]
             except KeyError:
                 namespace = get_namespace(name) or self.target_namespace
                 if namespace == XSI_NAMESPACE:
@@ -693,7 +685,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                         xsd_attribute = self.maps.lookup_attribute(name)
                     except LookupError:
                         try:
-                            xsd_attribute = self[None]  # None key ==> anyAttribute
+                            xsd_attribute = self._attribute_group[None]  # None == anyAttribute
                             value = (name, value)
                         except KeyError:
                             reason = "%r is not an attribute of the XSI namespace." % name
@@ -701,7 +693,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                             continue
                 else:
                     try:
-                        xsd_attribute = self[None]  # None key ==> anyAttribute
+                        xsd_attribute = self._attribute_group[None]  # None == anyAttribute
                         value = (name, value)
                     except KeyError:
                         reason = "%r attribute not allowed for element." % name
