@@ -12,26 +12,29 @@ This module contains classes for XML Schema elements, complex types and model gr
 """
 import warnings
 from decimal import Decimal
+from types import GeneratorType
 from typing import Optional
 from elementpath import XPath2Parser, ElementPathError, XPathContext
-from elementpath.datatypes import AbstractDateTime, Duration
+from elementpath.datatypes import AbstractDateTime, Duration, AbstractBinary
 
 from ..exceptions import XMLSchemaTypeError, XMLSchemaValueError
 from ..names import XSD_COMPLEX_TYPE, XSD_SIMPLE_TYPE, XSD_ALTERNATIVE, \
     XSD_ELEMENT, XSD_ANY_TYPE, XSD_UNIQUE, XSD_KEY, XSD_KEYREF, XSI_NIL, \
     XSI_TYPE, XSD_ERROR, XSD_NOTATION_TYPE
 from ..etree import etree_element
-from ..helpers import get_qname, get_namespace, etree_iter_location_hints
+from ..helpers import get_qname, get_namespace, etree_iter_location_hints, \
+    raw_xml_encode, strictly_equal
+from .. import dataobjects
 from ..converters import ElementData, XMLSchemaConverter
-from ..xpath import XMLSchemaProxy, ElementPathMixin
+from ..xpath import XMLSchemaProxy, ElementPathMixin, XPathElement
 
 from .exceptions import XMLSchemaValidationError, XMLSchemaTypeTableWarning
-from .helpers import get_xsd_derivation_attribute, raw_xml_encode, strictly_equal
+from .helpers import get_xsd_derivation_attribute
 from .xsdbase import XSD_TYPE_DERIVATIONS, XSD_ELEMENT_DERIVATIONS, \
     XsdComponent, XsdType, ValidationMixin
 from .particles import ParticleMixin
 from .models import OccursCounter
-from .identities import XsdKeyref
+from .identities import IdentityXPathContext, XsdIdentity, XsdKeyref
 from .wildcards import XsdAnyElement
 
 
@@ -77,9 +80,7 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
     _final = None
     _head_type = None
 
-    def __init__(self, elem, schema, parent):
-        super(XsdElement, self).__init__(elem, schema, parent)
-        ElementPathMixin.__init__(self)
+    binding = None
 
     def __repr__(self):
         return '%s(%s=%r, occurs=%r)' % (
@@ -375,6 +376,23 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
             return self._block
         return self.schema.block_default
 
+    def get_binding(self, *bases, replace_existing=False, **attrs):
+        """
+        Gets data object binding for XSD element, creating a new one if it doesn't exist.
+
+        :param bases: base classes to use for creating the binding class.
+        :param replace_existing: provide `True` to replace an existing binding class.
+        :param attrs: attribute and method definitions for the binding class body.
+        """
+        if self.binding is None or replace_existing:
+            if not bases:
+                bases = (dataobjects.DataElement,)
+            attrs['xsd_element'] = self
+            class_name = '{}Binding'.format(self.local_name.title().replace('_', ''))
+            self.binding = dataobjects.DataBindingMeta(class_name, bases, attrs)
+
+        return self.binding
+
     def get_attribute(self, name):
         if name[0] != '{':
             return self.type.attributes[get_qname(self.type.target_namespace, name)]
@@ -421,9 +439,8 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
         else:
             if isinstance(self, xsd_classes):
                 yield self
-            for obj in self.identities.values():
-                if isinstance(obj, xsd_classes):
-                    yield obj
+            if issubclass(XsdIdentity, xsd_classes):
+                yield from self.identities.values()
 
         if self.ref is None and self.type.parent is not None:
             yield from self.type.iter_components(xsd_classes)
@@ -540,7 +557,7 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
         except KeyError:
             converter = kwargs['converter'] = self.schema.get_converter(**kwargs)
         else:
-            if not isinstance(converter, XMLSchemaConverter) and converter is not None:
+            if converter is not None and not isinstance(converter, XMLSchemaConverter):
                 converter = kwargs['converter'] = self.schema.get_converter(**kwargs)
 
         try:
@@ -560,6 +577,14 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
                 xsd_type = self.maps.get_instance_type(type_name, xsd_type, namespaces)
             except (KeyError, TypeError) as err:
                 yield self.validation_error(validation, err, elem, **kwargs)
+            else:
+                if self.identities:
+                    xpath_element = XPathElement(self.name, xsd_type)
+                    for identity in self.identities.values():
+                        context = IdentityXPathContext(self.schema, item=xpath_element)
+                        for e in identity.selector.token.select_results(context):
+                            if e not in identity.elements:
+                                identity.elements[e] = None
 
             if xsd_type.is_blocked(self):
                 reason = "usage of %r is blocked" % xsd_type
@@ -608,7 +633,7 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
             else:
                 nilled = True
 
-        if xsd_type.is_empty() and elem.text:
+        if xsd_type.is_empty() and elem.text and xsd_type.normalize(elem.text):
             reason = "character data is not allowed because content is empty"
             yield self.validation_error(validation, reason, elem, **kwargs)
 
@@ -630,7 +655,7 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
 
             if self.fixed is not None and \
                     (len(elem) > 0 or value is not None and self.fixed != value):
-                reason = "must have the fixed value %r." % self.fixed
+                reason = "must have the fixed value %r" % self.fixed
                 yield self.validation_error(validation, reason, elem, **kwargs)
 
         else:
@@ -646,7 +671,7 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
                     pass
                 elif not strictly_equal(xsd_type.text_decode(text),
                                         xsd_type.text_decode(self.fixed)):
-                    reason = "must have the fixed value %r." % self.fixed
+                    reason = "must have the fixed value %r" % self.fixed
                     yield self.validation_error(validation, reason, elem, **kwargs)
 
             elif not text and self.default is not None and kwargs.get('use_defaults'):
@@ -686,20 +711,24 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
                     else:
                         value = result
 
-            if isinstance(value, Decimal):
+            if 'value_hook' in kwargs:
+                value = kwargs['value_hook'](value, xsd_type)
+            elif isinstance(value, (int, float, list)) or value is None:
+                pass
+            elif isinstance(value, str):
+                if value.startswith('{') and xsd_type.is_qname():
+                    value = text
+            elif isinstance(value, Decimal):
                 try:
                     value = kwargs['decimal_type'](value)
                 except (KeyError, TypeError):
                     pass
             elif isinstance(value, (AbstractDateTime, Duration)):
-                try:
-                    if kwargs['datetime_types'] is not True:
-                        value = elem.text
-                except KeyError:
-                    value = elem.text
-            elif isinstance(value, str) and value.startswith('{'):
-                if xsd_type.is_qname():
-                    value = text
+                if not kwargs.get('datetime_types'):
+                    value = str(value) if text is None else text.strip()
+            elif isinstance(value, AbstractBinary):
+                if not kwargs.get('binary_types'):
+                    value = str(value)
 
         if converter is not None:
             element_data = ElementData(elem.tag, value, content, attributes)
@@ -744,6 +773,17 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
                     except ValueError as err:
                         yield self.validation_error(validation, err, elem, **kwargs)
 
+        # Apply non XSD optional validations
+        if 'extra_validator' in kwargs:
+            try:
+                result = kwargs['extra_validator'](elem, self)
+            except XMLSchemaValidationError as err:
+                yield self.validation_error(validation, err, elem, **kwargs)
+            else:
+                if isinstance(result, GeneratorType):
+                    for err in result:
+                        yield self.validation_error(validation, err, elem, **kwargs)
+
         # Disable collect for out of scope identities and check key references
         if 'max_depth' not in kwargs:
             for identity in self.identities.values():
@@ -765,6 +805,8 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
         :return: yields an Element, eventually preceded by a sequence of \
         validation or encoding errors.
         """
+        errors = []
+
         try:
             converter = kwargs['converter']
         except KeyError:
@@ -777,10 +819,17 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
             level = kwargs['level']
         except KeyError:
             level = 0
+            element_data = converter.element_encode(obj, self, level)
+            if not self.is_matching(element_data.tag, self.default_namespace):
+                errors.append("data tag does not match XSD element name")
+
+            if 'max_depth' in kwargs and kwargs['max_depth'] == 0:
+                for e in errors:
+                    yield self.validation_error(validation, e, **kwargs)
+                return
+        else:
+            element_data = converter.element_encode(obj, self, level)
 
-        element_data = converter.element_encode(obj, self, level)
-        errors = []
-        tag = element_data.tag
         text = None
         children = element_data.content
         attributes = ()
@@ -872,7 +921,7 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
                 elif result:
                     text, children = result
 
-        elem = converter.etree_element(tag, text, children, attributes, level)
+        elem = converter.etree_element(element_data.tag, text, children, attributes, level)
 
         if errors:
             for e in errors:
@@ -881,7 +930,9 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
         del element_data
 
     def is_matching(self, name, default_namespace=None, group=None):
-        if default_namespace and name[0] != '{':
+        if not name:
+            return False
+        elif default_namespace and name[0] != '{':
             qname = '{%s}%s' % (default_namespace, name)
             if name == self.name or qname == self.name:
                 return True
@@ -892,7 +943,9 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
             return any(name == e.name for e in self.iter_substitutes())
 
     def match(self, name, default_namespace=None, **kwargs):
-        if default_namespace and name[0] != '{':
+        if not name:
+            return
+        elif default_namespace and name[0] != '{':
             qname = '{%s}%s' % (default_namespace, name)
             if name == self.name or qname == self.name:
                 return self
@@ -1015,6 +1068,9 @@ class XsdElement(XsdComponent, ValidationMixin, ParticleMixin, ElementPathMixin)
         except AttributeError:
             return True
 
+    def is_empty(self):
+        return self.fixed == '' or self.type.is_empty()
+
 
 class Xsd11Element(XsdElement):
     """
@@ -1097,6 +1153,7 @@ class Xsd11Element(XsdElement):
         else:
             if isinstance(self, xsd_classes):
                 yield self
+
             for obj in self.identities.values():
                 if isinstance(obj, xsd_classes):
                     yield obj
