@@ -375,28 +375,35 @@ class XsdGlobals(XsdValidator):
             errors.extend(schema.all_errors)
         return errors
 
+    def create_bindings(self, *bases, **attrs):
+        """Creates data object bindings for the XSD elements of built schemas."""
+        for xsd_element in self.iter_components(xsd_classes=XsdElement):
+            if xsd_element.target_namespace != XSD_NAMESPACE:
+                xsd_element.get_binding(*bases, replace_existing=True, **attrs)
+
+    def clear_bindings(self):
+        for xsd_element in self.iter_components(xsd_classes=XsdElement):
+            xsd_element.binding = None
+
     def iter_components(self, xsd_classes=None):
+        """Creates an iterator for the XSD components of built schemas."""
         if xsd_classes is None or isinstance(self, xsd_classes):
             yield self
         for xsd_global in self.iter_globals():
             yield from xsd_global.iter_components(xsd_classes)
 
-    def iter_schemas(self):
-        """Creates an iterator for the schemas registered in the instance."""
-        for schemas in self.namespaces.values():
-            yield from schemas
-
     def iter_globals(self):
-        """
-        Creates an iterator for XSD global definitions/declarations.
-        """
+        """Creates an iterator for the XSD global components of built schemas."""
         for global_map in self.global_maps:
             yield from global_map.values()
 
+    def iter_schemas(self):
+        """Creates an iterator for the registered schemas."""
+        for schemas in self.namespaces.values():
+            yield from schemas
+
     def register(self, schema):
-        """
-        Registers an XMLSchema instance.
-        """
+        """Registers an XMLSchema instance."""
         try:
             ns_schemas = self.namespaces[schema.target_namespace]
         except KeyError:
@@ -404,7 +411,10 @@ class XsdGlobals(XsdValidator):
         else:
             if schema in ns_schemas:
                 return
-            elif not any(schema.url == obj.url and schema.__class__ == obj.__class__
+            elif schema.url is None:
+                # only by multi-source init or add_schema() by user initiative
+                ns_schemas.append(schema)
+            elif not any(schema.url == obj.url and schema.__class__ is obj.__class__
                          for obj in ns_schemas):
                 ns_schemas.append(schema)
 
@@ -509,35 +519,29 @@ class XsdGlobals(XsdValidator):
         Build the maps of XSD global definitions/declarations. The global maps are
         updated adding and building the globals of not built registered schemas.
         """
+
         try:
             meta_schema = self.namespaces[XSD_NAMESPACE][0]
         except KeyError:
-            # Meta-schemas are not registered. If any of base namespaces is already registered
-            # create a new meta-schema, otherwise register the meta-schemas.
-            meta_schema = self.validator.meta_schema
-            if meta_schema is None:
-                raise XMLSchemaValueError("{!r} has not a meta-schema".format(self.validator))
+            if self.validator.meta_schema is None:
+                msg = "missing XSD namespace in meta-schema instance {!r}"
+                raise XMLSchemaValueError(msg.format(self.validator))
+            meta_schema = None
 
-            if any(ns in self.namespaces for ns in meta_schema.BASE_SCHEMAS):
-                base_schemas = {
-                    k: v for k, v in meta_schema.BASE_SCHEMAS.items() if k not in self.namespaces
-                }
-                meta_schema = self.validator.create_meta_schema(meta_schema.url, base_schemas, self)
-                for schema in self.iter_schemas():
-                    if schema.meta_schema is not None:
-                        schema.meta_schema = meta_schema
-            else:
-                for schema in meta_schema.maps.iter_schemas():
-                    self.register(schema)
+        if meta_schema is None or meta_schema.meta_schema is not None:
+            # XSD namespace not imported: replace the meta-schema with a new one
+            url = self.validator.meta_schema.url
+            base_schemas = {k: v for k, v in self.validator.meta_schema.BASE_SCHEMAS.items()
+                            if k not in self.namespaces}
+            meta_schema = self.validator.create_meta_schema(url, base_schemas, self)
 
-                self.types.update(meta_schema.maps.types)
-                self.attributes.update(meta_schema.maps.attributes)
-                self.attribute_groups.update(meta_schema.maps.attribute_groups)
-                self.groups.update(meta_schema.maps.groups)
-                self.notations.update(meta_schema.maps.notations)
-                self.elements.update(meta_schema.maps.elements)
-                self.substitution_groups.update(meta_schema.maps.substitution_groups)
-                self.identities.update(meta_schema.maps.identities)
+            for schema in self.iter_schemas():
+                if schema.meta_schema is not None:
+                    schema.meta_schema = meta_schema
+        else:
+            if not self.types and meta_schema.maps is not self:
+                for source_map, target_map in zip(meta_schema.maps.global_maps, self.global_maps):
+                    target_map.update(source_map)
 
         not_built_schemas = [schema for schema in self.iter_schemas() if not schema.built]
         for schema in not_built_schemas:
