@@ -19,7 +19,6 @@ import logging
 import threading
 import warnings
 import re
-import sys
 from copy import copy
 from abc import ABCMeta
 from collections import namedtuple, Counter
@@ -60,10 +59,6 @@ from .wildcards import XsdAnyElement, XsdAnyAttribute, Xsd11AnyElement, \
 from .global_maps import XsdGlobals
 
 logger = logging.getLogger('xmlschema')
-logging_formatter = logging.Formatter('[%(levelname)s] %(message)s')
-logging_handler = logging.StreamHandler(sys.stderr)
-logging_handler.setFormatter(logging_formatter)
-logger.addHandler(logging_handler)
 
 XSD_VERSION_PATTERN = re.compile(r'^\d+\.\d+$')
 DRIVE_PATTERN = re.compile(r'^[a-zA-Z]:$')
@@ -98,7 +93,6 @@ class XMLSchemaMeta(ABCMeta):
             # Defining a subclass without a meta-schema (eg. XMLSchemaBase)
             return super(XMLSchemaMeta, mcs).__new__(mcs, name, bases, dict_)
         dict_['meta_schema'] = None
-        dict_['lock'] = threading.Lock()  # Lock instance for shared meta-schemas
 
         xsd_version = dict_.get('XSD_VERSION') or get_attribute('XSD_VERSION', *bases)
         if xsd_version not in ('1.0', '1.1'):
@@ -135,7 +129,6 @@ class XMLSchemaMeta(ABCMeta):
         schema_location = meta_schema.url if isinstance(meta_schema, XMLSchemaBase) else meta_schema
         meta_schema = meta_schema_class.create_meta_schema(schema_location)
         dict_['meta_schema'] = meta_schema
-        dict_.pop('lock')
 
         return super(XMLSchemaMeta, mcs).__new__(mcs, name, bases, dict_)
 
@@ -145,10 +138,13 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
     Base class for an XML Schema instance.
 
     :param source: an URI that reference to a resource or a file path or a file-like \
-    object or a string containing the schema or an Element or an ElementTree document.
+    object or a string containing the schema or an Element or an ElementTree document \
+    or an :class:`XMLRecource` instance. A multi source initialization is supported \
+    providing a not empty list of XSD sources.
     :type source: Element or ElementTree or str or file-like object
-    :param namespace: is an optional argument that contains the URI of the namespace. \
-    When specified it must be equal to the *targetNamespace* declared in the schema.
+    :param namespace: is an optional argument that contains the URI of the namespace \
+    that has to used in case the schema has no namespace (chameleon schema). For other \
+    cases, when specified, it must be equal to the *targetNamespace* of the schema.
     :type namespace: str or None
     :param validation: the XSD validation mode to use for build the schema, \
     that can be 'strict' (default), 'lax' or 'skip'.
@@ -290,13 +286,15 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
     default_attributes = None
     default_open_content = None
     override = None
+
+    # Store XPath constructors tokens (for schema and its assertions)
     xpath_tokens = None
 
     def __init__(self, source, namespace=None, validation='strict', global_maps=None,
                  converter=None, locations=None, base_url=None, allow='all', defuse='remote',
                  timeout=300, build=True, use_meta=True, use_fallback=True, loglevel=None):
         super(XMLSchemaBase, self).__init__(validation)
-        ElementPathMixin.__init__(self)
+        self.lock = threading.Lock()  # Lock for build operations
 
         if loglevel is not None:
             if isinstance(loglevel, str):
@@ -313,6 +311,13 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             # Allow sandbox mode without a base_url using the initial schema URL as base
             base_url = os.path.dirname(normalize_url(source))
 
+        if not isinstance(source, list):
+            other_sources = None
+        elif not source:
+            raise XMLSchemaValueError("no XSD source provided!")
+        else:
+            source, other_sources = source[0], source[1:]
+
         if isinstance(source, XMLResource):
             self.source = source
         else:
@@ -405,16 +410,14 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
                 self.meta_schema.maps.build()
 
         # Create or set the XSD global maps instance
-        if global_maps is None:
-            if use_meta and self.target_namespace not in self.meta_schema.maps.namespaces:
-                self.maps = self.meta_schema.maps.copy(self, validation=validation)
-            else:
-                self.maps = XsdGlobals(self, validation)
-
-        elif isinstance(global_maps, XsdGlobals):
+        if isinstance(global_maps, XsdGlobals):
             self.maps = global_maps
+        elif global_maps is not None:
+            raise XMLSchemaTypeError("'global_maps' argument must be an %r instance" % XsdGlobals)
+        elif use_meta and self.target_namespace not in self.meta_schema.maps.namespaces:
+            self.maps = self.meta_schema.maps.copy(self, validation)
         else:
-            raise XMLSchemaTypeError("'global_maps' argument must be an %r instance." % XsdGlobals)
+            self.maps = XsdGlobals(self, validation)
 
         if any(ns == VC_NAMESPACE for ns in self.namespaces.values()):
             # For XSD 1.1+ apply versioning filter to schema tree. See the paragraph
@@ -438,10 +441,6 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             if ns not in self.maps.namespaces:
                 self._import_namespace(ns, self.locations[ns])
 
-        # Imports missing base namespaces
-        for ns in filter(lambda x: x not in self.maps.namespaces, self.BASE_SCHEMAS):
-            self._import_namespace(ns, [self.BASE_SCHEMAS[ns]])
-
         if '' not in self.namespaces:
             self.namespaces[''] = ''  # For default local names are mapped to no namespace
 
@@ -460,6 +459,17 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
                     self.default_open_content = XsdDefaultOpenContent(child, self)
                     break
 
+        if other_sources:
+            for _source in other_sources:
+                if not isinstance(_source, XMLResource):
+                    _source = XMLResource(_source, base_url, allow, defuse, timeout)
+
+                if not _source.root.get('targetNamespace') and self.target_namespace:
+                    # Adding a chameleon schema: set the namespace with targetNamespace
+                    self.add_schema(_source, namespace=self.target_namespace)
+                else:
+                    self.add_schema(_source)
+
         try:
             if build:
                 self.maps.build()
@@ -467,9 +477,15 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             if loglevel is not None:
                 logger.setLevel(logging.WARNING)  # Restore default logging
 
-    @property
-    def name(self):
-        return os.path.basename(self.url) if self.url else None
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state.pop('lock', None)
+        state.pop('xpath_tokens', None)
+        return state
+
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        self.lock = threading.Lock()
 
     def __repr__(self):
         if self.url:
@@ -517,7 +533,7 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
 
     @property
     def xsd_version(self):
-        """Property that returns the class attribute XSD_VERSION."""
+        """Compatibility property that returns the class attribute XSD_VERSION."""
         return self.XSD_VERSION
 
     # XML resource attributes access
@@ -530,9 +546,14 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         """Returns the source text of the XSD schema."""
         return self.source.get_text()
 
+    @property
+    def name(self):
+        """Schema resource name, is `None` if the schema is built from an Element or a string."""
+        return self.source.name
+
     @property
     def url(self):
-        """Schema resource URL, is `None` if the schema is built from a string."""
+        """Schema resource URL, is `None` if the schema is built from an Element or a string."""
         return self.source.url
 
     @property
@@ -540,6 +561,11 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         """The base URL of the source of the schema."""
         return self.source.base_url
 
+    @property
+    def filepath(self):
+        """The filepath if the schema is loaded from a local XSD file, `None` otherwise."""
+        return self.source.filepath
+
     @property
     def allow(self):
         """Defines the resource access security mode, can be 'all', 'local' or 'sandbox'."""
@@ -557,8 +583,8 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
 
     @property
     def use_meta(self):
-        """Returns `True` if the meta-schema is imported."""
-        return self.meta_schema is not None and XSD_NAMESPACE in self.maps.namespaces
+        """Returns `True` if the class meta-schema is used."""
+        return self.meta_schema is self.__class__.meta_schema
 
     # Schema root attributes
     @property
@@ -776,14 +802,16 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             any_type.attributes[None].maps = self.maps
         return any_type
 
-    def create_element(self, name, text=None, **attrib):
+    def create_element(self, name, parent=None, text=None, **attrib):
         """
-        Creates an xs:element instance related to schema instance.
+        Creates an xs:element instance related to schema component.
+        Used as dummy element for validation/decoding/encoding
+        operations of wildcards and complex types.
         """
         elem = etree_element(XSD_ELEMENT, name=name, **attrib)
         if text is not None:
             elem.text = text
-        return self.BUILDERS.element_class(elem=elem, schema=self, parent=None)
+        return self.BUILDERS.element_class(elem=elem, schema=self, parent=parent)
 
     def copy(self):
         """
@@ -965,6 +993,16 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         else:
             return self.find(path, namespaces)
 
+    def create_bindings(self, *bases, **attrs):
+        """
+        Creates data object bindings for XSD elements of the schema.
+
+        :param bases: base classes to use for creating the binding classes.
+        :param attrs: attribute and method definitions for the binding classes body.
+        """
+        for xsd_element in self.iter_components(xsd_classes=XsdElement):
+            xsd_element.get_binding(*bases, replace_existing=True, **attrs)
+
     def _parse_inclusions(self):
         """Processes schema document inclusions and redefinitions."""
         for child in self.source.root:
@@ -1018,12 +1056,13 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
                 else:
                     schema.redefine = self
 
-    def include_schema(self, location, base_url=None):
+    def include_schema(self, location, base_url=None, build=False):
         """
         Includes a schema for the same namespace, from a specific URL.
 
         :param location: is the URL of the schema.
         :param base_url: is an optional base URL for fetching the schema resource.
+        :param build: defines when to build the imported schema, the default is to not build.
         :return: the included :class:`XMLSchema` instance.
         """
         schema_url = fetch_resource(location, base_url)
@@ -1043,7 +1082,7 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
                 allow=self.allow,
                 defuse=self.defuse,
                 timeout=self.timeout,
-                build=False,
+                build=build,
             )
 
         if schema is self:
@@ -1191,14 +1230,39 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         self.imports[namespace] = schema
         return schema
 
-    def export(self, target, only_relative=True):
+    def add_schema(self, source, namespace=None, build=False):
+        """
+        Add another schema source to the maps of the instance.
+
+        :param source: an URI that reference to a resource or a file path or a file-like \
+        object or a string containing the schema or an Element or an ElementTree document.
+        :param namespace: is an optional argument that contains the URI of the namespace \
+        that has to used in case the schema has no namespace (chameleon schema). For other \
+        cases, when specified, it must be equal to the *targetNamespace* of the schema.
+        :param build: defines when to build the imported schema, the default is to not build.
+        :return: the added :class:`XMLSchema` instance.
+        """
+        return type(self)(
+            source=source,
+            namespace=namespace,
+            validation=self.validation,
+            global_maps=self.maps,
+            converter=self.converter,
+            locations=self._locations,
+            base_url=self.base_url,
+            allow=self.allow,
+            defuse=self.defuse,
+            timeout=self.timeout,
+            build=build,
+        )
+
+    def export(self, target, save_remote=False):
         """
         Exports a schema instance. The schema instance is exported to a
         directory with also the hierarchy of imported/included schemas.
 
         :param target: a path to a local empty directory.
-        :param only_relative: for default only loaded schemas referred by a relative \
-        location are saved. If `False` is provided all the loaded schemas are saved.
+        :param save_remote: if `True` is provided saves also remote schemas.
         """
         import pathlib
         from urllib.parse import urlsplit
@@ -1219,56 +1283,74 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
 
         url = self.url or 'schema.xsd'
         basename = pathlib.Path(urlsplit(url).path).name
-        exported_schemas = {self: [target_path.joinpath(basename), self.get_text()]}
+        exports = {self: [target_path.joinpath(basename), self.get_text()]}
 
         while True:
-            current_length = len(exported_schemas)
-            for schema in list(exported_schemas):
-                for location, ref_schema in \
-                        chain(schema.includes.items(), schema.imports.items()):
+            current_length = len(exports)
 
-                    if ref_schema in exported_schemas:
+            for schema in list(exports):
+                dir_path = exports[schema][0].parent
+                imports_items = [(x.url, x) for x in schema.imports.values() if x is not None]
+
+                for location, ref_schema in chain(schema.includes.items(), imports_items):
+                    if ref_schema in exports:
                         continue
 
-                    schema_path = exported_schemas[schema][0].parent
-
                     if is_remote_url(location):
-                        if only_relative:
+                        if not save_remote:
                             continue
                         url_parts = urlsplit(location)
                         netloc, path = url_parts.netloc, url_parts.path
                         path = pathlib.Path().joinpath(netloc).joinpath(path.lstrip('/'))
                     else:
+                        if location.startswith('file:/'):
+                            location = urlsplit(location).path
+
                         path = pathlib.Path(location)
                         if path.is_absolute():
-                            if only_relative:
-                                continue
+                            location = '/'.join(path.parts[-2:])
+                            try:
+                                schema_path = pathlib.Path(schema.filepath)
+                            except TypeError:
+                                pass
+                            else:
+                                try:
+                                    path = path.relative_to(schema_path.parent)
+                                except ValueError:
+                                    parts = path.parts
+                                    if parts[:-2] == schema_path.parts[:-2]:
+                                        path = pathlib.Path(location)
+                                else:
+                                    path = dir_path.joinpath(path)
+                                    exports[ref_schema] = [path, ref_schema.get_text()]
+                                    continue
+
                         elif not str(path).startswith('..'):
-                            path = schema_path.joinpath(path)
-                            exported_schemas[ref_schema] = [path, ref_schema.get_text()]
+                            path = dir_path.joinpath(path)
+                            exports[ref_schema] = [path, ref_schema.get_text()]
                             continue
 
-                    if DRIVE_PATTERN.match(path.parts[0]):
-                        path = pathlib.Path().joinpath(path.parts[1:])
+                        if DRIVE_PATTERN.match(path.parts[0]):
+                            path = pathlib.Path().joinpath(path.parts[1:])
 
-                    for strip_path in ('/', '\\', '..'):
-                        while True:
-                            try:
-                                path = path.relative_to(strip_path)
-                            except ValueError:
-                                break
+                        for strip_path in ('/', '\\', '..'):
+                            while True:
+                                try:
+                                    path = path.relative_to(strip_path)
+                                except ValueError:
+                                    break
 
                     path = target_path.joinpath(path)
                     repl = 'schemaLocation="{}"'.format(path.as_posix())
-                    text = exported_schemas[schema][1]
+                    schema_text = exports[schema][1]
                     pattern = r'\bschemaLocation\s*=\s*[\'\"].*%s.*[\'"]' % re.escape(location)
-                    exported_schemas[schema][1] = re.sub(pattern, repl, text)
-                    exported_schemas[ref_schema] = [path, ref_schema.get_text()]
+                    exports[schema][1] = re.sub(pattern, repl, schema_text)
+                    exports[ref_schema] = [path, ref_schema.get_text()]
 
-            if current_length == len(exported_schemas):
+            if current_length == len(exports):
                 break
 
-        for schema, (path, text) in exported_schemas.items():
+        for schema, (path, text) in exports.items():
             if not path.parent.exists():
                 path.parent.mkdir(parents=True)
 
@@ -1409,28 +1491,11 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             )
         return '{%s}%s' % (namespace, local_name)
 
-    def validate(self, source, path=None, schema_path=None, use_defaults=True, namespaces=None):
+    def validate(self, source, path=None, schema_path=None, use_defaults=True,
+                 namespaces=None, max_depth=None, extra_validator=None):
         """
         Validates an XML data against the XSD schema/component instance.
 
-        :raises: :exc:`XMLSchemaValidationError` if XML *data* instance is not a valid.
-        """
-        for error in self.iter_errors(source, path, schema_path, use_defaults, namespaces):
-            raise error
-
-    def is_valid(self, source, path=None, schema_path=None, use_defaults=True, namespaces=None):
-        """
-        Like :meth:`validate` except that do not raises an exception but returns ``True`` if
-        the XML data is valid, ``False`` if it's invalid.
-        """
-        error = next(self.iter_errors(source, path, schema_path, use_defaults, namespaces), None)
-        return error is None
-
-    def iter_errors(self, source, path=None, schema_path=None, use_defaults=True, namespaces=None):
-        """
-        Creates an iterator for the errors generated by the validation of an XML data
-        against the XSD schema/component instance.
-
         :param source: the source of XML data. Can be an :class:`XMLResource` instance, a \
         path to a file or an URI of a resource or an opened file-like object or an Element \
         instance or an ElementTree instance or a string containing the XML data.
@@ -1441,6 +1506,34 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         global element of the schema.
         :param use_defaults: Use schema's default values for filling missing data.
         :param namespaces: is an optional mapping from namespace prefix to URI.
+        :param max_depth: maximum level of validation, for default there is no limit. \
+        With lazy resources is set to `source.lazy_depth` for managing lazy validation.
+        :param extra_validator: an optional function for performing non-standard \
+        validations on XML data. The provided function is called for each traversed \
+        element, with the XML element as 1st argument and the corresponding XSD \
+        element as 2nd argument. It can be also a generator function and has to \
+        raise/yield :exc:`XMLSchemaValidationError` exceptions.
+        :raises: :exc:`XMLSchemaValidationError` if the XML data instance is invalid.
+        """
+        for error in self.iter_errors(source, path, schema_path, use_defaults,
+                                      namespaces, max_depth, extra_validator):
+            raise error
+
+    def is_valid(self, source, path=None, schema_path=None, use_defaults=True,
+                 namespaces=None, max_depth=None, extra_validator=None):
+        """
+        Like :meth:`validate` except that does not raise an exception but returns
+        ``True`` if the XML data instance is valid, ``False`` if it is invalid.
+        """
+        error = next(self.iter_errors(source, path, schema_path, use_defaults,
+                                      namespaces, max_depth, extra_validator), None)
+        return error is None
+
+    def iter_errors(self, source, path=None, schema_path=None, use_defaults=True,
+                    namespaces=None, max_depth=None, extra_validator=None):
+        """
+        Creates an iterator for the errors generated by the validation of an XML data against
+        the XSD schema/component instance. Accepts the same arguments of :meth:`validate`.
         """
         self.check_validator(validation='lax')
         if not isinstance(source, XMLResource):
@@ -1471,6 +1564,10 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             'inherited': {},
             'locations': locations,  # TODO: lazy schemas load
         }
+        if max_depth is not None:
+            kwargs['max_depth'] = max_depth
+        if extra_validator is not None:
+            kwargs['extra_validator'] = extra_validator
 
         if path:
             selector = source.iterfind(path, namespaces, nsmap=namespaces, ancestors=ancestors)
@@ -1568,9 +1665,9 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
 
     def iter_decode(self, source, path=None, schema_path=None, validation='lax',
                     process_namespaces=True, namespaces=None, use_defaults=True,
-                    decimal_type=None, datetime_types=False, converter=None,
-                    filler=None, fill_missing=False, keep_unknown=False,
-                    max_depth=None, depth_filler=None, **kwargs):
+                    decimal_type=None, datetime_types=False, binary_types=False,
+                    converter=None, filler=None, fill_missing=False, keep_unknown=False,
+                    max_depth=None, depth_filler=None, value_hook=None, **kwargs):
         """
         Creates an iterator for decoding an XML source to a data structure.
 
@@ -1584,7 +1681,7 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         global element of the schema.
         :param validation: defines the XSD validation mode to use for decode, can be \
         'strict', 'lax' or 'skip'.
-        :param process_namespaces: whether to use namespace information in the
+        :param process_namespaces: whether to use namespace information in the \
         decoding process, using the map provided with the argument *namespaces* \
         and the map extracted from the XML document.
         :param namespaces: is an optional mapping from namespace prefix to URI.
@@ -1593,7 +1690,9 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         `xs:decimal` built-in and derived types), useful if you want to generate a \
         JSON-compatible data structure.
         :param datetime_types: if set to `True` the datetime and duration XSD types \
-        are decoded, otherwise their origin XML string is returned.
+        are kept decoded, otherwise their origin XML string is returned.
+        :param binary_types: if set to `True` xs:hexBinary and xs:base64Binary types \
+        are kept decoded, otherwise their origin XML string is returned.
         :param converter: an :class:`XMLSchemaConverter` subclass or instance to use \
         for decoding.
         :param filler: an optional callback function to fill undecodable data with a \
@@ -1604,10 +1703,14 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         The filling value is `None` or a typed value if the *filler* callback is provided.
         :param keep_unknown: if set to `True` unknown tags are kept and are decoded with \
         *xs:anyType*. For default unknown tags not decoded by a wildcard are discarded.
-        :param max_depth: maximum level of decoding, for default there is no limit.
+        :param max_depth: maximum level of decoding, for default there is no limit. \
+        With lazy resources is set to `source.lazy_depth` for managing lazy decoding.
         :param depth_filler: an optional callback function to replace data over the \
         *max_depth* level. The callback function must accept one positional argument, that \
         can be an XSD Element. If not provided deeper data are replaced with `None` values.
+        :param value_hook: an optional function that will be called with any decoded \
+        atomic value and the XSD type used for decoding. The return value will be used \
+        instead of the original value.
         :param kwargs: keyword arguments with other options for converter and decoder.
         :return: yields a decoded data object, eventually preceded by a sequence of \
         validation or decoding errors.
@@ -1631,7 +1734,6 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             namespaces=namespaces,
             source=source,
             use_defaults=use_defaults,
-            datetime_types=datetime_types,
             id_map=Counter(),
             identities={},
             inherited={},
@@ -1639,6 +1741,10 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
 
         if decimal_type is not None:
             kwargs['decimal_type'] = decimal_type
+        if datetime_types:
+            kwargs['datetime_types'] = datetime_types
+        if binary_types:
+            kwargs['binary_types'] = binary_types
         if filler is not None:
             kwargs['filler'] = filler
         if fill_missing:
@@ -1649,6 +1755,8 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             kwargs['max_depth'] = max_depth
         if depth_filler is not None:
             kwargs['depth_filler'] = depth_filler
+        if value_hook is not None:
+            kwargs['value_hook'] = value_hook
 
         if path:
             selector = source.iterfind(path, namespaces, nsmap=namespaces)
@@ -1718,7 +1826,7 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         :param unordered: a flag for explicitly activating unordered encoding mode for \
         content model data. This mode uses content models for a reordered-by-model \
         iteration of the child elements.
-        :param kwargs: Keyword arguments containing options for converter and encoding.
+        :param kwargs: keyword arguments containing options for converter.
         :return: yields an Element instance/s or validation/encoding errors.
         """
         self.check_validator(validation)
