@@ -11,13 +11,15 @@
 This module contains base functions and classes XML Schema components.
 """
 import re
+from typing import Optional
 
 from ..exceptions import XMLSchemaValueError, XMLSchemaTypeError
-from ..qnames import XSD_ANNOTATION, XSD_APPINFO, XSD_DOCUMENTATION, XML_LANG, \
+from ..names import XSD_ANNOTATION, XSD_APPINFO, XSD_DOCUMENTATION, XML_LANG, \
     XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XSD_ID, XSD_QNAME, \
-    XSD_OVERRIDE, XSD_NOTATION_TYPE, get_qname, local_name, get_prefixed_qname, \
-    is_not_xsd_annotation
+    XSD_OVERRIDE, XSD_NOTATION_TYPE, XSD_DECIMAL
 from ..etree import is_etree_element, etree_tostring
+from ..helpers import get_qname, local_name, get_prefixed_qname
+from .. import dataobjects
 from .exceptions import XMLSchemaParseError, XMLSchemaValidationError
 
 XSD_TYPE_DERIVATIONS = {'extension', 'restriction'}
@@ -36,7 +38,7 @@ def check_validation_mode(validation):
                                   "'lax' or 'skip': %r" % validation)
 
 
-class XsdValidator(object):
+class XsdValidator:
     """
     Common base class for XML Schema validator, that represents a PSVI (Post Schema Validation
     Infoset) information item. A concrete XSD validator have to report its validity collecting
@@ -51,15 +53,12 @@ class XsdValidator(object):
     :ivar errors: XSD validator building errors.
     :vartype errors: list
     """
-    xsd_version = None
+    xsd_version = elem = None
 
     def __init__(self, validation='strict'):
         self.validation = validation
         self.errors = []
 
-    def __str__(self):
-        return self.__repr__()
-
     @property
     def built(self):
         """
@@ -143,12 +142,9 @@ class XsdValidator(object):
 
         if validation == 'skip':
             return
-
-        if is_etree_element(elem):
-            pass
         elif elem is None:
-            elem = getattr(self, 'elem', None)
-        else:
+            elem = self.elem
+        elif not is_etree_element(elem):
             msg = "the argument 'elem' must be an Element instance, not {!r}."
             raise XMLSchemaTypeError(msg.format(elem))
 
@@ -221,13 +217,13 @@ class XsdComponent(XsdValidator):
     parent = None
     name = None
     ref = None
+    annotation = None
     qualified = True
+    redefine = None
 
-    def __init__(self, elem, schema, parent=None, name=None):
+    def __init__(self, elem, schema, parent=None, name: Optional[str] = None):
         super(XsdComponent, self).__init__(schema.validation)
-        if name is not None:
-            assert name and (name[0] == '{' or not schema.target_namespace), \
-                "name=%r argument: must be a qualified name of the target namespace." % name
+        if name:
             self.name = name
         if parent is not None:
             self.parent = parent
@@ -236,28 +232,14 @@ class XsdComponent(XsdValidator):
         self.elem = elem
 
     def __setattr__(self, name, value):
-        if name == "elem":
-            if not is_etree_element(value):
-                raise XMLSchemaTypeError(
-                    "%r attribute must be an Etree Element: %r" % (name, value)
-                )
-            elif value.tag not in self._ADMITTED_TAGS:
-                raise XMLSchemaValueError(
-                    "wrong XSD element %r for %r, must be one of %r." % (
-                        local_name(value.tag), self,
-                        [local_name(tag) for tag in self._ADMITTED_TAGS]
-                    )
-                )
-            super(XsdComponent, self).__setattr__(name, value)
-            self._parse()
-            return
-        elif name == "schema":
-            if hasattr(self, 'schema') and self.schema.target_namespace != value.target_namespace:
-                raise XMLSchemaValueError(
-                    "cannot change 'schema' attribute of %r: the actual %r has a different "
-                    "target namespace than %r." % (self, self.schema, value)
-                )
         super(XsdComponent, self).__setattr__(name, value)
+        if name == 'elem':
+            if value.tag not in self._ADMITTED_TAGS:
+                msg = "wrong XSD element {!r} for {!r}, must be one of {!r}"
+                raise XMLSchemaValueError(
+                    msg.format(value.tag, self.__class__, self._ADMITTED_TAGS)
+                )
+            self._parse()
 
     @property
     def xsd_version(self):
@@ -314,22 +296,19 @@ class XsdComponent(XsdValidator):
         return self.maps.types[XSD_ANY_ATOMIC_TYPE]
 
     def __repr__(self):
-        if self.name is None:
-            return '<%s at %#x>' % (self.__class__.__name__, id(self))
-        elif self.ref is not None:
+        if self.ref is not None:
             return '%s(ref=%r)' % (self.__class__.__name__, self.prefixed_name)
         else:
             return '%s(name=%r)' % (self.__class__.__name__, self.prefixed_name)
 
     def _parse(self):
         del self.errors[:]
-        try:
-            if self.elem[0].tag == XSD_ANNOTATION:
-                self.annotation = XsdAnnotation(self.elem[0], self.schema, self)
-            else:
-                self.annotation = None
-        except (TypeError, IndexError):
-            self.annotation = None
+        for child in self.elem:
+            if child.tag == XSD_ANNOTATION:
+                self.annotation = XsdAnnotation(child, self.schema, self)
+                break
+            elif not callable(child.tag):
+                break
 
     def _parse_reference(self):
         """
@@ -358,32 +337,24 @@ class XsdComponent(XsdValidator):
             except (KeyError, ValueError, RuntimeError) as err:
                 self.parse_error(err)
             else:
-                if self._parse_child_component(self.elem) is not None:
+                if self._parse_child_component(self.elem, strict=False) is not None:
                     self.parse_error("a reference component cannot have "
                                      "child definitions/declarations")
                 return True
 
-    def _parse_boolean_attribute(self, name):
-        try:
-            value = self.elem.attrib[name].strip()
-        except KeyError:
-            return
-        else:
-            if value in ('true', '1'):
-                return True
-            elif value in ('false', '0'):
-                return False
-            else:
-                self.parse_error("wrong value %r for boolean attribute %r" % (value, name))
-
     def _parse_child_component(self, elem, strict=True):
         child = None
-        for index, child in enumerate(filter(is_not_xsd_annotation, elem)):
-            if not strict:
-                return child
-            elif index:
+        for e in elem:
+            if e.tag == XSD_ANNOTATION or callable(e.tag):
+                continue
+            elif not strict:
+                return e
+            elif child is not None:
                 msg = "too many XSD components, unexpected {!r} found at position {}"
-                self.parse_error(msg.format(child, index), elem)
+                self.parse_error(msg.format(child, elem[:].index(e)), elem)
+                break
+            else:
+                child = e
         return child
 
     def _parse_target_namespace(self):
@@ -472,6 +443,39 @@ class XsdComponent(XsdValidator):
         """
         return self if self.is_matching(name, default_namespace, **kwargs) else None
 
+    def get_matching_item(self, mapping, ns_prefix='xmlns', match_local_name=False):
+        """
+        If a key is matching component name, returns its value, otherwise returns `None`.
+        """
+        if self.name is None:
+            return
+        elif not self.target_namespace:
+            return mapping.get(self.name)
+        elif self.qualified_name in mapping:
+            return mapping[self.qualified_name]
+        elif self.prefixed_name in mapping:
+            return mapping[self.prefixed_name]
+
+        # Try a match with other prefixes
+        target_namespace = self.target_namespace
+        suffix = ':%s' % self.local_name
+
+        for k in filter(lambda x: x.endswith(suffix), mapping):
+            prefix = k.split(':')[0]
+            if self.namespaces.get(prefix) == target_namespace:
+                return mapping[k]
+
+            # Match namespace declaration within value
+            ns_declaration = '{}:{}'.format(ns_prefix, prefix)
+            try:
+                if mapping[k][ns_declaration] == target_namespace:
+                    return mapping[k]
+            except (KeyError, TypeError):
+                pass
+        else:
+            if match_local_name:
+                return mapping.get(self.local_name)
+
     def get_global(self):
         """Returns the global XSD component that contains the component instance."""
         if self.parent is None:
@@ -579,7 +583,6 @@ class XsdType(XsdComponent):
     block = None
     base_type = None
     derivation = None
-    redefine = None
     _final = None
 
     @property
@@ -647,7 +650,6 @@ class XsdType(XsdComponent):
     @staticmethod
     def is_complex():
         """Returns `True` if the instance is a complexType, `False` otherwise."""
-        raise NotImplementedError()
 
     @staticmethod
     def is_atomic():
@@ -683,14 +685,12 @@ class XsdType(XsdComponent):
         """
         Returns `True` if the instance has a simple content, `False` otherwise.
         """
-        raise NotImplementedError()
 
     def has_complex_content(self):
         """
         Returns `True` if the instance is a complexType with mixed or element-only
         content, `False` otherwise.
         """
-        raise NotImplementedError()
 
     def has_mixed_content(self):
         """
@@ -706,6 +706,11 @@ class XsdType(XsdComponent):
         raise NotImplementedError()
 
     def is_derived(self, other, derivation=None):
+        """
+        Returns `True` if the instance is derived from *other*, `False` otherwise.
+        The optional argument derivation can be a string containing the words
+        'extension' or 'restriction' or both.
+        """
         raise NotImplementedError()
 
     def is_extension(self):
@@ -743,59 +748,67 @@ class XsdType(XsdComponent):
     def is_notation(self):
         return self.name == XSD_NOTATION_TYPE or self.is_derived(self.maps.types[XSD_NOTATION_TYPE])
 
+    def is_decimal(self):
+        return self.name == XSD_DECIMAL or self.is_derived(self.maps.types[XSD_DECIMAL])
+
     def text_decode(self, text):
         raise NotImplementedError()
 
 
-class ValidationMixin(object):
+class ValidationMixin:
     """
     Mixin for implementing XML data validators/decoders. A derived class must implement the
     methods `iter_decode` and `iter_encode`.
     """
-    def validate(self, source, use_defaults=True, namespaces=None):
+    def validate(self, source, use_defaults=True, namespaces=None,
+                 max_depth=None, extra_validator=None):
         """
-        Validates an XML data against the XSD schema/component instance.
+        Validates XML data against the XSD schema/component instance.
 
-        :param source: the source of XML data. For a schema can be a path \
+        :param source: the source of the XML data. For a schema can be a path \
         to a file or an URI of a resource or an opened file-like object or an Element Tree \
         instance or a string containing XML data. For other XSD components can be a string \
         for an attribute or a simple type validators, or an ElementTree's Element otherwise.
         :param use_defaults: indicates whether to use default values for filling missing data.
         :param namespaces: is an optional mapping from namespace prefix to URI.
-        :raises: :exc:`XMLSchemaValidationError` if XML *data* instance is not a valid.
+        :param max_depth: maximum level of validation, for default there is no limit.
+        :param extra_validator: an optional function for performing non-standard \
+        validations on XML data. The provided function is called for each traversed \
+        element, with the XML element as 1st argument and the corresponding XSD \
+        element as 2nd argument. It can be also a generator function and has to \
+        raise/yield :exc:`XMLSchemaValidationError` exceptions.
+        :raises: :exc:`XMLSchemaValidationError` if the XML data instance is invalid.
         """
-        for error in self.iter_errors(source, use_defaults=use_defaults, namespaces=namespaces):
+        for error in self.iter_errors(source, use_defaults, namespaces,
+                                      max_depth, extra_validator):
             raise error
 
-    def is_valid(self, source, use_defaults=True, namespaces=None):
+    def is_valid(self, source, use_defaults=True, namespaces=None,
+                 max_depth=None, extra_validator=None):
         """
-        Like :meth:`validate` except that do not raises an exception but returns ``True`` if
-        the XML document is valid, ``False`` if it's invalid.
+        Like :meth:`validate` except that does not raise an exception but returns
+        ``True`` if the XML data instance is valid, ``False`` if it is invalid.
+        """
+        error = next(self.iter_errors(source, use_defaults, namespaces,
+                                      max_depth, extra_validator), None)
+        return error is None
 
-        :param source: the source of XML data. For a schema can be a path \
-        to a file or an URI of a resource or an opened file-like object or an Element Tree \
-        instance or a string containing XML data. For other XSD components can be a string \
-        for an attribute or a simple type validators, or an ElementTree's Element otherwise.
-        :param use_defaults: indicates whether to use default values for filling missing data.
-        :param namespaces: is an optional mapping from namespace prefix to URI.
+    def iter_errors(self, source, use_defaults=True, namespaces=None,
+                    max_depth=None, extra_validator=None):
         """
-        return next(
-            self.iter_errors(source, use_defaults=use_defaults, namespaces=namespaces), None
-        ) is None
+        Creates an iterator for the errors generated by the validation of an XML data against
+        the XSD schema/component instance. Accepts the same arguments of :meth:`validate`.
+        """
+        kwargs = {
+            'use_defaults': use_defaults,
+            'namespaces': namespaces,
+        }
+        if max_depth is not None:
+            kwargs['max_depth'] = max_depth
+        if extra_validator is not None:
+            kwargs['extra_validator'] = extra_validator
 
-    def iter_errors(self, source, use_defaults=True, namespaces=None):
-        """
-        Creates an iterator for the errors generated by the validation of an XML data
-        against the XSD schema/component instance.
-
-        :param source: the source of XML data. For a schema can be a path \
-        to a file or an URI of a resource or an opened file-like object or an Element Tree \
-        instance or a string containing XML data. For other XSD components can be a string \
-        for an attribute or a simple type validators, or an ElementTree's Element otherwise.
-        :param use_defaults: Use schema's default values for filling missing data.
-        :param namespaces: is an optional mapping from namespace prefix to URI.
-        """
-        for result in self.iter_decode(source, use_defaults=use_defaults, namespaces=namespaces):
+        for result in self.iter_decode(source, **kwargs):
             if isinstance(result, XMLSchemaValidationError):
                 yield result
             else:
@@ -831,6 +844,24 @@ class ValidationMixin(object):
 
         return (result, errors) if validation == 'lax' else result
 
+    def to_objects(self, source, with_bindings=False, **kwargs):
+        """
+        Decodes XML data to Python data objects.
+
+        :param source: the XML data. Can be a string for an attribute or for a simple \
+        type components or a dictionary for an attribute group or an ElementTree's \
+        Element for other components.
+        :param with_bindings: if `True` is provided the decoding is done using \
+        :class:`DataBindingConverter` that used XML data binding classes. For \
+        default the objects are intances of :class:`DataElement` and uses the \
+        :class:`DataElementConverter`.
+        :param kwargs: other optional keyword arguments for the method \
+        :func:`iter_decode`, except the argument *converter*.
+        """
+        if with_bindings:
+            return self.decode(source, converter=dataobjects.DataBindingConverter, **kwargs)
+        return self.decode(source, converter=dataobjects.DataElementConverter, **kwargs)
+
     def encode(self, obj, validation='strict', **kwargs):
         """
         Encodes data to XML.
@@ -846,7 +877,6 @@ class ValidationMixin(object):
         component, or also if it's invalid when ``validation='strict'`` is provided.
         """
         check_validation_mode(validation)
-
         result, errors = None, []
         for result in self.iter_encode(obj, validation=validation, **kwargs):  # pragma: no cover
             if not isinstance(result, XMLSchemaValidationError):
@@ -872,7 +902,7 @@ class ValidationMixin(object):
 
     def iter_encode(self, obj, validation='lax', **kwargs):
         """
-        Creates an iterator for Encode data to an Element.
+        Creates an iterator for encoding data to an Element tree.
 
         :param obj: The data that has to be encoded.
         :param validation: The validation mode. Can be 'lax', 'strict' or 'skip'.
@@ -913,115 +943,3 @@ class ValidationMixin(object):
         if validation == 'strict' and error.elem is not None:
             raise error
         return error
-
-
-class ParticleMixin(object):
-    """
-    Mixin for objects related to XSD Particle Schema Components:
-
-      https://www.w3.org/TR/2012/REC-xmlschema11-1-20120405/structures.html#p
-      https://www.w3.org/TR/2012/REC-xmlschema11-1-20120405/structures.html#t
-
-    :ivar min_occurs: the minOccurs property of the XSD particle. Defaults to 1.
-    :ivar max_occurs: the maxOccurs property of the XSD particle. Defaults to 1, \
-    a `None` value means 'unbounded'.
-    """
-    min_occurs = 1
-    max_occurs = 1
-
-    @property
-    def occurs(self):
-        return [self.min_occurs, self.max_occurs]
-
-    @property
-    def effective_min_occurs(self):
-        return self.min_occurs
-
-    @property
-    def effective_max_occurs(self):
-        return self.max_occurs
-
-    def is_emptiable(self):
-        """
-        Tests if max_occurs == 0. A zero-length model group is considered emptiable.
-        For model groups the test outcome depends also on nested particles.
-        """
-        return self.min_occurs == 0
-
-    def is_empty(self):
-        """
-        Tests if max_occurs == 0. A zero-length model group is considered empty.
-        """
-        return self.max_occurs == 0
-
-    def is_single(self):
-        """
-        Tests if the particle has max_occurs == 1. For elements the test
-        outcome depends also on parent group. For model groups the test
-        outcome depends also on nested model groups.
-        """
-        return self.max_occurs == 1
-
-    def is_multiple(self):
-        """Tests the particle can have multiple occurrences."""
-        return not self.is_empty() and not self.is_single()
-
-    def is_ambiguous(self):
-        """Tests if min_occurs != max_occurs."""
-        return self.min_occurs != self.max_occurs
-
-    def is_univocal(self):
-        """Tests if min_occurs == max_occurs."""
-        return self.min_occurs == self.max_occurs
-
-    def is_missing(self, occurs):
-        """Tests if provided occurrences are under the minimum."""
-        return not self.is_emptiable() if occurs == 0 else self.min_occurs > occurs
-
-    def is_over(self, occurs):
-        """Tests if provided occurrences are over the maximum."""
-        return self.max_occurs is not None and self.max_occurs <= occurs
-
-    def has_occurs_restriction(self, other):
-        if self.min_occurs < other.min_occurs:
-            return False
-        elif self.max_occurs == 0:
-            return True
-        elif other.max_occurs is None:
-            return True
-        elif self.max_occurs is None:
-            return False
-        else:
-            return self.max_occurs <= other.max_occurs
-
-    def parse_error(self, message):
-        raise XMLSchemaParseError(self, message)   # pragma: no cover
-
-    def _parse_particle(self, elem):
-        if 'minOccurs' in elem.attrib:
-            try:
-                min_occurs = int(elem.attrib['minOccurs'])
-            except (TypeError, ValueError):
-                self.parse_error("minOccurs value is not an integer value")
-            else:
-                if min_occurs < 0:
-                    self.parse_error("minOccurs value must be a non negative integer")
-                else:
-                    self.min_occurs = min_occurs
-
-        max_occurs = elem.get('maxOccurs')
-        if max_occurs is None:
-            if self.min_occurs > 1:
-                self.parse_error("minOccurs must be lesser or equal than maxOccurs")
-        elif max_occurs == 'unbounded':
-            self.max_occurs = None
-        else:
-            try:
-                max_occurs = int(max_occurs)
-            except ValueError:
-                self.parse_error("maxOccurs value must be a non negative integer or 'unbounded'")
-            else:
-                if self.min_occurs > max_occurs:
-                    self.parse_error("maxOccurs must be 'unbounded' or greater than minOccurs")
-                else:
-                    self.max_occurs = max_occurs
