@@ -13,12 +13,12 @@ This module defines a proxy class and a mixin class for enabling XPath on schema
 from abc import abstractmethod
 from collections.abc import Sequence
 import re
-import threading
 
 from elementpath import AttributeNode, TypedElement, XPath2Parser, \
     XPathSchemaContext, AbstractSchemaProxy
 
 from .names import XSD_NAMESPACE
+from .helpers import get_qname, local_name, get_prefixed_qname
 from .exceptions import XMLSchemaValueError, XMLSchemaTypeError
 
 _REGEX_TAG_POSITION = re.compile(r'\b\[\d+]')
@@ -88,22 +88,18 @@ class XMLSchemaProxy(AbstractSchemaProxy):
             except AttributeError:
                 raise XMLSchemaTypeError("%r is not an XsdElement" % base_element)
 
-    @property
-    def xsd_version(self):
-        return self._schema.XSD_VERSION
-
     def bind_parser(self, parser):
-        if parser.schema is not self:
-            parser.schema = self
+        parser.schema = self
+        parser.symbol_table = parser.__class__.symbol_table.copy()
 
-        if self._schema.xpath_tokens is None:
-            parser.symbol_table = parser.__class__.symbol_table.copy()
-            for xsd_type in self.iter_atomic_types():
-                parser.schema_constructor(xsd_type.name)
-            self._schema.xpath_tokens = parser.symbol_table
-        else:
-            parser.symbol_table = self._schema.xpath_tokens
-        parser.tokenizer = parser.create_tokenizer(parser.symbol_table)
+        with self._schema.lock:
+            if self._schema.xpath_tokens is None:
+                self._schema.xpath_tokens = {
+                    xsd_type.name: parser.schema_constructor(xsd_type.name)
+                    for xsd_type in self.iter_atomic_types()
+                }
+
+        parser.symbol_table.update(self._schema.xpath_tokens)
 
     def get_context(self):
         return XMLSchemaContext(
@@ -112,34 +108,8 @@ class XMLSchemaProxy(AbstractSchemaProxy):
             item=self._base_element
         )
 
-    def get_type(self, qname):
-        try:
-            return self._schema.maps.types[qname]
-        except KeyError:
-            return None
-
-    def get_attribute(self, qname):
-        try:
-            return self._schema.maps.attributes[qname]
-        except KeyError:
-            return None
-
-    def get_element(self, qname):
-        try:
-            return self._schema.maps.elements[qname]
-        except KeyError:
-            return None
-
-    def get_substitution_group(self, qname):
-        try:
-            return self._schema.maps.substitution_groups[qname]
-        except KeyError:
-            return None
-
-    def find(self, path, namespaces=None):
-        return self._schema.find(path, namespaces)
-
     def is_instance(self, obj, type_qname):
+        # FIXME: use elementpath.datatypes for checking atomic datatypes
         xsd_type = self._schema.maps.types[type_qname]
         try:
             xsd_type.encode(obj)
@@ -163,7 +133,7 @@ class XMLSchemaProxy(AbstractSchemaProxy):
 
 class ElementPathMixin(Sequence):
     """
-    Mixin abstract class for enabling ElementTree and XPath API on XSD components.
+    Mixin abstract class for enabling ElementTree and XPath 2.0 API on XSD components.
 
     :cvar text: the Element text, for compatibility with the ElementTree API.
     :cvar tail: the Element tail, for compatibility with the ElementTree API.
@@ -174,22 +144,6 @@ class ElementPathMixin(Sequence):
     namespaces = {}
     xpath_default_namespace = ''
 
-    _xpath_parser = None  # Internal XPath 2.0 parser, instantiated at first use.
-
-    def __init__(self):
-        self._xpath_lock = threading.Lock()  # Lock for XPath operations
-
-    def __getstate__(self):
-        state = self.__dict__.copy()
-        state.pop('_xpath_lock', None)
-        state.pop('_xpath_parser', None)
-        state.pop('xpath_tokens', None)  # For schema objects
-        return state
-
-    def __setstate__(self, state):
-        self.__dict__.update(state)
-        self._xpath_lock = threading.Lock()
-
     @abstractmethod
     def __iter__(self):
         raise NotImplementedError
@@ -242,29 +196,20 @@ class ElementPathMixin(Sequence):
         xpath_namespaces.update(namespaces)
         return xpath_namespaces
 
-    def _xpath_parse(self, path, namespaces=None):
-        path = _REGEX_TAG_POSITION.sub('', path.strip())  # Strips tags positions from path
-
-        namespaces = self._get_xpath_namespaces(namespaces)
-        with self._xpath_lock:
-            parser = self._xpath_parser
-            if parser is None:
-                parser = XPath2Parser(namespaces, strict=False, schema=self.xpath_proxy)
-                self._xpath_parser = parser
-            else:
-                parser.namespaces = namespaces
-            return parser.parse(path)
-
     def find(self, path, namespaces=None):
         """
         Finds the first XSD subelement matching the path.
 
         :param path: an XPath expression that considers the XSD component as the root element.
         :param namespaces: an optional mapping from namespace prefix to namespace URI.
-        :return: The first matching XSD subelement or ``None`` if there is not match.
+        :return: the first matching XSD subelement or ``None`` if there is no match.
         """
+        path = _REGEX_TAG_POSITION.sub('', path.strip())  # Strips tags positions from path
+        namespaces = self._get_xpath_namespaces(namespaces)
+        parser = XPath2Parser(namespaces, strict=False)
         context = XMLSchemaContext(self)
-        return next(self._xpath_parse(path, namespaces).select_results(context), None)
+
+        return next(parser.parse(path).select_results(context), None)
 
     def findall(self, path, namespaces=None):
         """
@@ -275,8 +220,12 @@ class ElementPathMixin(Sequence):
         :return: a list containing all matching XSD subelements in document order, an empty \
         list is returned if there is no match.
         """
+        path = _REGEX_TAG_POSITION.sub('', path.strip())  # Strips tags positions from path
+        namespaces = self._get_xpath_namespaces(namespaces)
+        parser = XPath2Parser(namespaces, strict=False)
         context = XMLSchemaContext(self)
-        return self._xpath_parse(path, namespaces).get_results(context)
+
+        return parser.parse(path).get_results(context)
 
     def iterfind(self, path, namespaces=None):
         """
@@ -286,8 +235,12 @@ class ElementPathMixin(Sequence):
         :param namespaces: is an optional mapping from namespace prefix to full name.
         :return: an iterable yielding all matching XSD subelements in document order.
         """
+        path = _REGEX_TAG_POSITION.sub('', path.strip())  # Strips tags positions from path
+        namespaces = self._get_xpath_namespaces(namespaces)
+        parser = XPath2Parser(namespaces, strict=False)
         context = XMLSchemaContext(self)
-        return self._xpath_parse(path, namespaces).select_results(context)
+
+        return parser.parse(path).select_results(context)
 
     def iter(self, tag=None):
         """
@@ -324,3 +277,49 @@ class ElementPathMixin(Sequence):
         for child in self:
             if tag is None or child.is_matching(tag):
                 yield child
+
+
+class XPathElement(ElementPathMixin):
+    """An element node for making XPath operations on schema types."""
+
+    parent = None
+
+    def __init__(self, name, xsd_type):
+        self.name = name
+        self.type = xsd_type
+        try:
+            self.attributes = xsd_type.attributes
+        except AttributeError:
+            pass
+
+    def __iter__(self):
+        if not self.type.has_simple_content():
+            yield from self.type.content.iter_elements()
+
+    @property
+    def xpath_proxy(self):
+        return XMLSchemaProxy(self.schema, self)
+
+    @property
+    def schema(self):
+        return self.type.schema
+
+    @property
+    def target_namespace(self):
+        return self.type.schema.target_namespace
+
+    @property
+    def namespaces(self):
+        return self.type.schema.namespaces
+
+    @property
+    def local_name(self):
+        return local_name(self.name)
+
+    @property
+    def qualified_name(self):
+        return get_qname(self.target_namespace, self.name)
+
+    @property
+    def prefixed_name(self):
+        return get_prefixed_qname(self.name, self.namespaces)
